
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include <stdarg.h>
#include <fcntl.h>
#include <errno.h>
#include <private/android_filesystem_config.h>

char* mount_point = NULL;
int mount_point_len = 0;

void fixup_fs_config(char* folder, char* prefix, FILE* out_fp)
{
    DIR *d = NULL;
    struct dirent *de = NULL;
    int folder_path_len = 0;
    int prefix_len = 0;

    if ((NULL == folder) || (0 == (folder_path_len = strlen(folder))))
    {
        printf("Folder path cannot be null!\n");
        return;
    }

    if (NULL == out_fp)
    {
        printf("Output fp cannot be null!\n");
        return;
    }

    /* open the folder & parse all file */
    d = opendir(folder);

    if (NULL == d) {
        printf("Cannot open directory: %s\n", folder);
        return;
    }

    if (NULL != prefix)
    {
        prefix_len = strlen(prefix);
    }

    while (0 != (de = readdir(d)))
    {
        struct stat s;
        char entry[2048];
        char entry_w_prefix[2048];
        char entry_w_prefix_mount_point[2048];

        memset(entry, 0, sizeof(entry));
        memset(entry_w_prefix, 0, sizeof(entry_w_prefix));
        memset(entry_w_prefix_mount_point, 0, sizeof(entry_w_prefix_mount_point));
        memset(&s, 0, sizeof(struct stat));

        if('.' == de->d_name[0])
        {
            continue;
        }

        /* build up folder + entry full string */
        memcpy(entry, folder, folder_path_len + 1);
        entry[folder_path_len] = '/';
        memcpy(entry + folder_path_len + 1, de->d_name, strlen(de->d_name) + 1);

        /* build up mount point + entry full string */
        if (0 != prefix_len)
        {
            memcpy(entry_w_prefix, prefix, prefix_len + 1);
            entry_w_prefix[prefix_len] = '/';
            memcpy(entry_w_prefix + prefix_len + 1, de->d_name, strlen(de->d_name) + 1);
        }
        else
        {
            memcpy(entry_w_prefix, de->d_name, strlen(de->d_name) + 1);
        }

        /* build up mount_point + prefix + entry full string */
        memcpy(entry_w_prefix_mount_point, mount_point, mount_point_len + 1);
        entry_w_prefix_mount_point[mount_point_len] = '/';
        memcpy(entry_w_prefix_mount_point + mount_point_len + 1, entry_w_prefix, strlen(entry_w_prefix) + 1);

#if DEBUG
        printf("===================================\n");
        printf("Entry:%s\n", entry);
        printf("entry_w_prefix:%s\n", entry_w_prefix);
        printf("entry_w_prefix_mount_point:%s\n", entry_w_prefix_mount_point);
        printf("===================================\n");
#endif

        if(lstat(entry, &s))
        {
            printf("Could not stat: %s\n", entry);
            return;
        }

        /* fixup file system configurations */
        int isDIR = S_ISDIR(s.st_mode);
        unsigned mode = 0;
        unsigned uid = 0;
        unsigned gid = 0;
        uint64_t capabilities = 0;

        fs_config(entry_w_prefix_mount_point, isDIR, &uid, &gid, &mode, &capabilities);
        fprintf(out_fp, "%s m %o %d %d\n", entry_w_prefix, mode, uid, gid);

        /* fixup recuresivly */
        if(isDIR)
        {
            fixup_fs_config(entry, entry_w_prefix, out_fp);
        }
    }
    closedir(d);

    return;
}

int main(int argc, char *argv[])
{
    FILE* fp = NULL;

    if (4 != argc)
    {
        printf("Usage: %s <folder path for fixup file system configuration> <output file> <mount point>\n", argv[0]);
        return -1;
    }
    fp = fopen(argv[2], "w");

    if (NULL == fp)
    {
        printf("Cannot open file: %s\n", argv[2]);
        return -1;
    }

    if ((NULL == argv[3]) || (0 == (mount_point_len = strlen(argv[3]))))
    {
        printf("Mount point cannot be null!\n");
        return -1;
    }
    mount_point = argv[3];

    fixup_fs_config(argv[1], NULL, fp);

    fclose(fp);

    return 0;
}

