/*----------------------------------------------------------------------------*
 * Copyright Statement:                                                       *
 *                                                                            *
 *   This software/firmware and related documentation ("MediaTek Software")   *
 * are protected under international and related jurisdictions'copyright laws *
 * as unpublished works. The information contained herein is confidential and *
 * proprietary to MediaTek Inc. Without the prior written permission of       *
 * MediaTek Inc., any reproduction, modification, use or disclosure of        *
 * MediaTek Software, and information contained herein, in whole or in part,  *
 * shall be strictly prohibited.                                              *
 * MediaTek Inc. Copyright (C) 2010. All rights reserved.                     *
 *                                                                            *
 *   BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND     *
 * AGREES TO THE FOLLOWING:                                                   *
 *                                                                            *
 *   1)Any and all intellectual property rights (including without            *
 * limitation, patent, copyright, and trade secrets) in and to this           *
 * Software/firmware and related documentation ("MediaTek Software") shall    *
 * remain the exclusive property of MediaTek Inc. Any and all intellectual    *
 * property rights (including without limitation, patent, copyright, and      *
 * trade secrets) in and to any modifications and derivatives to MediaTek     *
 * Software, whoever made, shall also remain the exclusive property of        *
 * MediaTek Inc.  Nothing herein shall be construed as any transfer of any    *
 * title to any intellectual property right in MediaTek Software to Receiver. *
 *                                                                            *
 *   2)This MediaTek Software Receiver received from MediaTek Inc. and/or its *
 * representatives is provided to Receiver on an "AS IS" basis only.          *
 * MediaTek Inc. expressly disclaims all warranties, expressed or implied,    *
 * including but not limited to any implied warranties of merchantability,    *
 * non-infringement and fitness for a particular purpose and any warranties   *
 * arising out of course of performance, course of dealing or usage of trade. *
 * MediaTek Inc. does not provide any warranty whatsoever with respect to the *
 * software of any third party which may be used by, incorporated in, or      *
 * supplied with the MediaTek Software, and Receiver agrees to look only to   *
 * such third parties for any warranty claim relating thereto.  Receiver      *
 * expressly acknowledges that it is Receiver's sole responsibility to obtain *
 * from any third party all proper licenses contained in or delivered with    *
 * MediaTek Software.  MediaTek is not responsible for any MediaTek Software  *
 * releases made to Receiver's specifications or to conform to a particular   *
 * standard or open forum.                                                    *
 *                                                                            *
 *   3)Receiver further acknowledge that Receiver may, either presently       *
 * and/or in the future, instruct MediaTek Inc. to assist it in the           *
 * development and the implementation, in accordance with Receiver's designs, *
 * of certain softwares relating to Receiver's product(s) (the "Services").   *
 * Except as may be otherwise agreed to in writing, no warranties of any      *
 * kind, whether express or implied, are given by MediaTek Inc. with respect  *
 * to the Services provided, and the Services are provided on an "AS IS"      *
 * basis. Receiver further acknowledges that the Services may contain errors  *
 * that testing is important and it is solely responsible for fully testing   *
 * the Services and/or derivatives thereof before they are used, sublicensed  *
 * or distributed. Should there be any third party action brought against     *
 * MediaTek Inc. arising out of or relating to the Services, Receiver agree   *
 * to fully indemnify and hold MediaTek Inc. harmless.  If the parties        *
 * mutually agree to enter into or continue a business relationship or other  *
 * arrangement, the terms and conditions set forth herein shall remain        *
 * effective and, unless explicitly stated otherwise, shall prevail in the    *
 * event of a conflict in the terms in any agreements entered into between    *
 * the parties.                                                               *
 *                                                                            *
 *   4)Receiver's sole and exclusive remedy and MediaTek Inc.'s entire and    *
 * cumulative liability with respect to MediaTek Software released hereunder  *
 * will be, at MediaTek Inc.'s sole discretion, to replace or revise the      *
 * MediaTek Software at issue.                                                *
 *                                                                            *
 *   5)The transaction contemplated hereunder shall be construed in           *
 * accordance with the laws of Singapore, excluding its conflict of laws      *
 * principles.  Any disputes, controversies or claims arising thereof and     *
 * related thereto shall be settled via arbitration in Singapore, under the   *
 * then current rules of the International Chamber of Commerce (ICC).  The    *
 * arbitration shall be conducted in English. The awards of the arbitration   *
 * shall be final and binding upon both parties and shall be entered and      *
 * enforceable in any court of competent jurisdiction.                        *
 *---------------------------------------------------------------------------*/
/*-----------------------------------------------------------------------------
 * $RCSfile: push_play.c,v $
 * $Revision: #2 $
 *---------------------------------------------------------------------------*/
/** @file MMPlay.c
 *  This file contains sample codes for file-based MultiMedia Playback.
 */

//-----------------------------------------------------------------------------
// Include files
//-----------------------------------------------------------------------------

#include "mtimage.h"
#include "drv_common.h"
#include <stdio.h>
#include <stdlib.h>
#include <memory.h>
#include <pthread.h>
#include <errno.h>
#include "mtcommon.h"
#include "mtgfx.h"


#ifdef CC_SUPPORT_MTK_ANDROID_RM
#include "MtkResourceManager.h"
#include "include/RMClientPriority.h"
#endif

#include "jpeglib.h"
#include "jpgdec.h"

#define JPEG_DISPLAY_WIDTH          1920
#define JPEG_DISPLAY_HEIGHT         1080

#define NULL_MTIMAGE_HANDLE         ((MTIMAGE_HANDLE)0x200)

#define JPG_GETIMG_OSDMODE_ARGB8888     0
#define JPG_GETIMG_OSDMODE_ARGB565      1
#define JPG_GETIMG_OSDMODE_ARGB4444     2

//-----------------------------------------------------------------------------
// Static variable
//-----------------------------------------------------------------------------

#include <utils/Log.h>
#include <sys/time.h>

//-----------------------------------------------------------------------------
// Static functions
//-----------------------------------------------------------------------------
#if 0
UINT32 jpg_GetMSecs()
{
    struct timeval tv;
    gettimeofday(&tv, NULL);
    return (UINT32) (tv.tv_sec * 1000 + tv.tv_usec / 1000 ); // microseconds to milliseconds
}
#endif

static VOID _CallBack(
    MTIMAGE_CALLBACK_TYPE_T eCallbackType,
    UINT32 u4ErrorType,UINT32 u4Param)
{
    j_decompress_ptr cinfo = (j_decompress_ptr)u4Param;
    
    switch (eCallbackType)
    {
        case MT_IMAGE_CALLBACK_ERROR:      // notification of play error
            ALOGD("***ERROR.\n");
            break;
        case MT_IMAGE_CALLBACK_FINISH:
            ALOGD("***_CallBack EOF.\n");
            break;
        case MT_IMAGE_CALLBACK_NEEDDATA:
            ALOGD("***Need data\n");
            return;
        case MT_IMAGE_CALLBACK_STOPDONE:
            ALOGD("***STOP DONE.\n");
            break;
        case MT_IMAGE_CALLBACK_ALREADYSTOPED:
            ALOGD("***ALREADY STOPEED.\n");
            break;
        default:
            ALOGD("***callback error\n");
            return;
    }
    if (cinfo->client_data != NULL)
    {
        ((JPEG_HW_DECODER_T*)(cinfo->client_data))->img_play_status = (UINT8)eCallbackType;
    }
}

void jpeg_read(j_decompress_ptr cinfo)
{
    READ_BYTE(cinfo,return);
}

void jpeg_restart_sync(j_decompress_ptr cinfo)
{
    RESTART_BYTE(cinfo,return);
}



#define MAX_SEEK_BYTES 5*1024*1024

boolean jpeg_seek_ex(UINT32 u4Position, VOID* pvTag)
{
    j_decompress_ptr cinfo = (j_decompress_ptr)pvTag;

    ALOGD("{jpeg_seek_ex} u4Position=%d .\n", u4Position);

    if (cinfo->src->seek_input_data) 
    {
        return cinfo->src->seek_input_data(cinfo, u4Position);
    }

    return TRUE;
}
#define ONCE_READ_SIZE  (1*1024)
UINT32 jpeg_read_ex(UINT32 u4Addr,UINT32 u4Len, VOID* pvTag)
{
    j_decompress_ptr cinfo = (j_decompress_ptr)pvTag;
    UINT32 i,readtime;
    UINT32 u4TempAddr = u4Addr;
    UINT32 retLen = 0, leftLen = 0;
    JPEG_HW_DECODER_T* ptHwJpeg;

    if (cinfo->client_data == NULL)
    {
        return 0;
    }
    
    ptHwJpeg = (JPEG_HW_DECODER_T*)cinfo->client_data;

    ALOGD("jpg_read_ex needSize=0x%x, byte_in_buffer=0x%x.\n", 
        u4Len, cinfo->src->bytes_in_buffer);

    if (u4Len > cinfo->src->bytes_in_buffer)
    {
        if (cinfo->src->bytes_in_buffer != 0)
        {
            retLen += cinfo->src->bytes_in_buffer;
            memcpy((void*)u4TempAddr,cinfo->src->next_input_byte,cinfo->src->bytes_in_buffer);
            u4TempAddr+= cinfo->src->bytes_in_buffer;
            u4Len -= cinfo->src->bytes_in_buffer;
        }

        readtime =   u4Len / ONCE_READ_SIZE;

        for (i = 0; i < readtime;i++)
        {
            jpeg_read(cinfo);

            retLen += cinfo->src->bytes_in_buffer;
            memcpy((void*)u4TempAddr,cinfo->src->next_input_byte,cinfo->src->bytes_in_buffer);
            u4TempAddr+=cinfo->src->bytes_in_buffer;
            if (cinfo->src->bytes_in_buffer == ONCE_READ_SIZE)
            {
                cinfo->src->bytes_in_buffer = 0;
            }
            else
            {
                cinfo->src->bytes_in_buffer = 0;
                break;
            }
        }
        if (i == readtime)
        {
            leftLen = u4Len - readtime * ONCE_READ_SIZE;
            if (leftLen > 0)
            {
                jpeg_read(cinfo);

                if (cinfo->src->bytes_in_buffer >= leftLen)
                {
                    retLen += leftLen;
                    memcpy((void*)u4TempAddr,cinfo->src->next_input_byte,leftLen);
                    cinfo->src->bytes_in_buffer -= leftLen;
                    cinfo->src->next_input_byte += leftLen;
                }
                else
                {
                    retLen += cinfo->src->bytes_in_buffer;
                    memcpy((void*)u4TempAddr,cinfo->src->next_input_byte,cinfo->src->bytes_in_buffer);
                    cinfo->src->bytes_in_buffer = 0;
                }
            }
        }
    }
    else
    {
        retLen += u4Len;
        memcpy((void*)u4TempAddr,cinfo->src->next_input_byte,u4Len);
        cinfo->src->bytes_in_buffer -= u4Len;
        cinfo->src->next_input_byte += u4Len;
    }

    //fileSize limitation
    if (u4Len == 0x7ffc0 
        && (cinfo->src->current_offset < (500*1024)))
    {
        //ALOGD("filesize is smaller than 500K, cinfo->src->current_offset =0x%x.\n", cinfo->src->current_offset );
        //return 0;
    }

    return retLen;
}


static BOOL fgInited = FALSE;


boolean fgJpegIsFree(void)
{
    MTIMAGE_STATUS_T rStatus;
    MT_RESULT_T      tRet = MTR_OK;
    
    if (!fgInited)
    {
        MTAL_Init();
        fgInited =TRUE;
    }
    tRet = MTIMAGE_GetInfo(HW_JPG_DEC, NULL_MTIMAGE_HANDLE,&rStatus);
    if (tRet == MTR_OK)
    {
        ALOGD("MTIMAGE_GetInfo ok.\n");
        return TRUE;
    }
    ALOGD("MTIMAGE_GetInfo fail, ret=%d.\n", tRet);
    return FALSE;
}

void stop_jpeg_dec(j_decompress_ptr cinfo)
{

    MT_RESULT_T eImgRet = MTR_OK;
    JPEG_HW_DECODER_T* ptHwJpeg;

    if (cinfo == NULL || cinfo->client_data == NULL)
    {
        return;
    }

    ptHwJpeg = (JPEG_HW_DECODER_T*)cinfo->client_data;
    usleep(500);

    if((!fgJpegIsFree()) && (!ptHwJpeg->fg_hw_stop))
    {
        ptHwJpeg->fg_hw_stop = TRUE;
        MTIMAGE_SetStopFlag(HW_JPG_DEC, ptHwJpeg->fg_hw_stop);
        MTIMAGE_Stop(HW_JPG_DEC, NULL_MTIMAGE_HANDLE);
        
        pthread_mutex_lock(&ptHwJpeg->JpgStopMutex);
        ptHwJpeg->fg_waiting_stop = TRUE;
        pthread_cond_wait(&ptHwJpeg->JpgStopCond, &ptHwJpeg->JpgStopMutex);     //signal when stop finish
        ptHwJpeg->fg_waiting_stop = FALSE;
        ptHwJpeg->fg_hw_stop = FALSE;         
        MTIMAGE_SetStopFlag(HW_JPG_DEC,ptHwJpeg->fg_hw_stop);
        pthread_mutex_unlock(&ptHwJpeg->JpgStopMutex);
    }
    ALOGD("***stop finish..\n");
}


#ifdef CC_SUPPORT_MTK_ANDROID_RM

static void RmJpgClose(j_decompress_ptr cinfo)
{
    // release component & resource when exit.
    JPEG_HW_DECODER_T* ptHwJpeg;

    if (cinfo->client_data == NULL)
    {
        return;
    }
    
    ptHwJpeg = (JPEG_HW_DECODER_T*)cinfo->client_data;
    
    if (ptHwJpeg->HwJpgComp)
    {
        IResourceComponent* comp = (IResourceComponent*)ptHwJpeg->HwJpgComp;
        comp->ReleaseAllResource(comp);
        RMDestoryComponent(comp);
        ptHwJpeg->HwJpgComp = NULL;
    }
}

BOOL RmJpgRelease(void* ctx, void* pv_data)
{
    JPEG_HW_DECODER_T* ptHwJpeg;

    if (((j_decompress_ptr)ctx)->client_data == NULL)
    {
        return TRUE;
    }
    
    ptHwJpeg = (JPEG_HW_DECODER_T*)((j_decompress_ptr)ctx)->client_data;
    
    ALOGD("RmJpgRelease is called 111.\r\n");
    stop_jpeg_dec((j_decompress_ptr)ctx);
    ALOGD("RmJpgRelease is called 222.\r\n");
    if (ptHwJpeg->HwJpgComp)
    {
        IResourceComponent* comp = (IResourceComponent*)ptHwJpeg->HwJpgComp;
        comp->ReleaseAllResource(comp);
    }
    ALOGD("RmJpgRelease is called 333.\r\n");
    return TRUE;
}

static BOOL RmJpgOpen(j_decompress_ptr cinfo)
{
    JPEG_HW_DECODER_T* ptHwJpeg;
    IResourceComponent* comp = NULL;

    if (cinfo->client_data == NULL)
    {
        return FALSE;
    }

    ptHwJpeg = (JPEG_HW_DECODER_T*)cinfo->client_data;
    if(ptHwJpeg->HwJpgComp != NULL)
    {
        ALOGD("component != NULL, already alloc before \r\n");
        return TRUE;
    }
        
    Resource res;
    int         ret;
    res.e_Type = RESOURCE_TYPE_NUMBERNIC;
    res.resourceName = HWJPEGDECODER;
    res.u.NumbernicType.value = 0;
    // check resource either regist. if resource not regist, return error.
    // create component;
    ret = RMCreateComponent("JpegDecoder", RmJpgRelease, (void*)cinfo, &comp );
    if (ret != 0)
    {
        ALOGD("RMCreateComponent fail, ret=%d.\n", ret);
        return FALSE;
    }

    ret = RMSetComponentPriority(comp, RMClientPriority_SkiaLib);
    if (ret != 0)
    {
        RMDestoryComponent(comp);
        ALOGD("RMSetComponentPriority fail, ret=%d.\n", ret);
        return FALSE;
    }

    ptHwJpeg->HwJpgComp = comp;
    ALOGD("open: ptHwJpeg->HwJpgComp=0x%x.\n", ptHwJpeg->HwJpgComp);
    
    
    // check resource either regist. if resource not regist, return error.
    if (RMCheckResource(HWJPEGDECODER))
    {
        if (comp->AddResource(comp, &res))
        {
            if (comp->AllocResource(comp))
            {
                ALOGD("RmJpgOpen return OK.\n");    
                return TRUE;   
            }
        }
    }
    
    RmJpgClose(cinfo);   ////alloc resource failed, free all alloced resource by myself.
    
    return FALSE;
}

#endif

boolean mtimage_jpg_dec_close(j_decompress_ptr cinfo)
{
    JPEG_HW_DECODER_T* ptHwJpeg;

    if (cinfo->client_data == NULL)
    {
        return FALSE;
    }
    
    ptHwJpeg = (JPEG_HW_DECODER_T*)cinfo->client_data;
    MTIMAGE_Close(HW_JPG_DEC, NULL_MTIMAGE_HANDLE);
    ALOGD("***mtimage_jpg_dec_close...fgStop %d\n",ptHwJpeg->fg_hw_stop);
    //ALOGD("Getpid %d GetTid %d\n",getpid(),gettid());

    if((ptHwJpeg->fg_hw_stop) 
        || (ptHwJpeg->img_play_status == MT_IMAGE_CALLBACK_STOPDONE) 
        || (ptHwJpeg->img_play_status == MT_IMAGE_CALLBACK_ALREADYSTOPED))
    {
        ALOGD("***mtimage_jpg_dec_close stop lock\n");
        pthread_mutex_lock(&ptHwJpeg->JpgStopMutex);
        while(!ptHwJpeg->fg_waiting_stop)
        {
            ALOGD("***mtimage_jpg_dec_close while loop.....\n");
            pthread_mutex_unlock(&ptHwJpeg->JpgStopMutex);
            usleep(200);            
            pthread_mutex_lock(&ptHwJpeg->JpgStopMutex);
        }
        pthread_cond_signal(&ptHwJpeg->JpgStopCond);      
        pthread_mutex_unlock(&ptHwJpeg->JpgStopMutex);
        ALOGD("***mtimage_jpg_dec_close stop unlock\n");
    }

    MTIMAGE_Connect(HW_JPG_DEC, FALSE, MT_IMAGE_COLOR_FORMAT_ARGB8888);

#ifdef CC_SUPPORT_MTK_ANDROID_RM
    RmJpgClose();
#endif

    pthread_cond_destroy(&ptHwJpeg->JpgStopCond);
    pthread_mutex_destroy(&ptHwJpeg->JpgStopMutex);

    free(ptHwJpeg);
    cinfo->client_data = NULL;
    
    return TRUE;
}

BOOL mtmplay_jpg_dec(j_decompress_ptr cinfo)
{
    MTIMAGE_HANDLE      hMtimgHandle = NULL;
    MTIMAGE_SETTING_T   rMtimgSetting;
    MTIMAGE_STATUS_T rMtimgStatus;
    UINT32 i, u4Cnt = 0;
    BOOL    fgFileEnd =0;
    MT_RESULT_T eImgRet = MTR_OK;
    MTIMAGE_HANDLE pImageItem = NULL;
    UINT32 u4TgtWidth,u4TgtHeight;
    UINT8 *pTgtAddr;
    char *msg ;
    unsigned long mmuTable = 0;
    struct jpeg_source_mgr * datasrc = cinfo->src;
    JPEG_HW_DECODER_T* ptHwJpeg;

    if (cinfo->client_data == NULL)
    {
        return FALSE;
    }
    
    ptHwJpeg = (JPEG_HW_DECODER_T*)cinfo->client_data;

    ALOGD("{%s %d} pid=%d.\n", __FUNCTION__, __LINE__, getpid());

    if (!fgInited)
    {
        MTAL_Init();
        fgInited =TRUE;
    }

#ifdef CC_SUPPORT_MTK_ANDROID_RM
    if (!RmJpgOpen(cinfo))
    {
        ALOGD("RmJpgOpen return fail.\n");
        return FALSE;
    }
#endif
    
    ALOGD("***MTM_JPG_DEC 4.2 cinfo = 0x%p \n",cinfo);

    memset(&rMtimgSetting, 0, sizeof(MTIMAGE_SETTING_T));
    memset(&rMtimgStatus, 0, sizeof(MTIMAGE_STATUS_T));

    switch (cinfo->out_color_space)
    {
        case JCS_RGB:
        case JCS_RGBA_8888:
            rMtimgSetting.eColorFormat = MT_IMAGE_COLOR_FORMAT_ARGB8888; 
            ALOGD("***mtmplay_jpg_dec JCS_RGBA_8888\n");
            break;
        case JCS_RGB_565:
            rMtimgSetting.eColorFormat = MT_IMAGE_COLOR_FORMAT_ARGB565;
            ALOGD("***mtmplay_jpg_dec JCS_RGB_565\n");
            break;
        case JCS_YV16:
            rMtimgSetting.eColorFormat = MT_IMAGE_COLOR_FORMAT_YV16;
            ALOGD("***mtmplay_jpg_dec YV16\n");
            break;
        default:
            printf("Strange format for decoder\n");
            #ifdef CC_SUPPORT_MTK_ANDROID_RM
            RmJpgClose();
            #endif
            return FALSE;
    }
    
    //jpeg_restart_sync(cinfo); 
    if (cinfo->src->seek_input_data) 
    {
        if (cinfo->src->seek_input_data(cinfo, 0) == FALSE)
        {
            #ifdef CC_SUPPORT_MTK_ANDROID_RM
            RmJpgClose();
            #endif
            return FALSE;
        }
    }

    ALOGD("cinfo->output_width=%d, output_height=%d.\n", cinfo->output_width, cinfo->output_height);
    
    u4TgtWidth= (cinfo->output_width+0xf)&~0xf;
    u4TgtHeight= cinfo->output_height;

    ptHwJpeg->fg_hw_stop = FALSE;
    MTIMAGE_SetStopFlag(HW_JPG_DEC, ptHwJpeg->fg_hw_stop);

    if (MTIMAGE_Connect(HW_JPG_DEC, TRUE, rMtimgSetting.eColorFormat) != 0)
    {
        ALOGD("***mtmplay_jpg_dec MTIMAGE_Connect fail\n");
        #ifdef CC_SUPPORT_MTK_ANDROID_RM
        RmJpgClose();
        #endif
        return FALSE;
    }

    ALOGD("***mtmplay_jpg_dec MTIMAGE_DataFromBuffer\n");

    eImgRet = MTIMAGE_DataFromBuffer(HW_JPG_DEC, &pImageItem, NULL_MTIMAGE_HANDLE, datasrc->bytes_in_buffer, (UINT32)0);
    if (eImgRet != MTR_OK)
    {
        ALOGD("***MTIMAGE_DataFromBuffer fail %d.\n", eImgRet);
        mtimage_jpg_dec_close(cinfo);
        return FALSE;
    }

    MTIMAGE_RegCb(HW_JPG_DEC, pImageItem, _CallBack, (UINT32)cinfo); 
    MTIMAGE_RegReadFuc_Ex(HW_JPG_DEC, pImageItem, jpeg_read_ex, cinfo, NULL);
    //play
    ptHwJpeg->img_play_status = MT_IMAGE_CALLBACK_MAX;

    MTIMAGE_SetDecResolution(HW_JPG_DEC, NULL, u4TgtWidth, u4TgtHeight, 0);
    MTIMAGE_SetDecRatio(HW_JPG_DEC, NULL, cinfo->scale_denom);
    if (cinfo->out_color_space == JCS_YV16)
    {
        MTIMAGE_SetDecRatio(HW_JPG_DEC, NULL, 0);
        MTIMAGE_SetDecAddr(HW_JPG_DEC, NULL, 0x160);
    }

    ALOGD("***MTIMAGE_Start-----\n");

    if (MTIMAGE_Start(HW_JPG_DEC, pImageItem) != 0)
    {
        ALOGD("***MM Start play failed\n");
        mtimage_jpg_dec_close(cinfo);
        return FALSE;
    }
    
    // wait decode done
    MTIMAGE_Filldata_Libjpeg(HW_JPG_DEC);
    ALOGD("***MTIMAGE_Filldata_Libjpeg done.\n");

    if (ptHwJpeg->img_play_status != MT_IMAGE_CALLBACK_FINISH)
    {
        ALOGD("***MM Emp play status %d, return\n", ptHwJpeg->img_play_status);
        mtimage_jpg_dec_close(cinfo);
        return FALSE;
    }
    else
    {
        ALOGD("-------------MTIMAGE_decode finish-----\n");
    }

    if (MT_IMAGE_CALLBACK_ERROR != ptHwJpeg->img_play_status)
    {
        UINT32 u4Buf = 0, u4PhyBuf = 0,u4CbCrOffset = 0;
        MTVDO_REGION_T rRegion;

        MTIMAGE_GetImg(HW_JPG_DEC, hMtimgHandle, &u4Buf, &u4PhyBuf,&u4CbCrOffset,&rRegion, JPG_GETIMG_OSDMODE_ARGB8888,FALSE);
    
        cinfo->output_width = rRegion.u4Width;
        cinfo->output_height = rRegion.u4Height;
        cinfo->input_iMCU_row = (JDIMENSION)u4Buf ;
        ptHwJpeg->CBuffer = u4CbCrOffset;

        ALOGD("***0. output_width = %d, output_height = %d, outoutput buffer=%x\n", cinfo->output_width,  cinfo->output_height, u4Buf);
        ALOGD("***->MTM_JPG_DEC-DONE\n");
        return TRUE;
    }
    else
    {
        ALOGD("***->MTM_JPG_DEC-FAILE\n");
        mtimage_jpg_dec_close(cinfo);
        return FALSE;
    }
}

boolean jpg_init(j_decompress_ptr cinfo)
{
    JPEG_HW_DECODER_T* ptHwJpeg = NULL;

    ptHwJpeg = (JPEG_HW_DECODER_T*)malloc(sizeof(JPEG_HW_DECODER_T));
    if (ptHwJpeg == NULL)
    {
        ALOGD("{%s %d} malloc fail.\n", __FUNCTION__, __LINE__);
        return FALSE;
    }

    ptHwJpeg->magic_number = JPEG_HW_MAGIC_NUMBER;
    cinfo->client_data = (void*)ptHwJpeg;

    ptHwJpeg->fg_force_hw_mode = TRUE;

    return TRUE;
}

int jpg_decode(j_decompress_ptr cinfo)
{
    JDIMENSION  output_pitch;
    INT32       ret;
    JPEG_HW_DECODER_T* ptHwJpeg = NULL;
    boolean        bUseHwDecoder = FALSE;
    static boolean fgJpgMutexInit = FALSE;
    static pthread_mutex_t JpgLibMutex ;

    if (cinfo->client_data != NULL
        && *((INT32*)cinfo->client_data) == JPEG_HW_MAGIC_NUMBER)
    {
        ptHwJpeg = (JPEG_HW_DECODER_T*)(cinfo->client_data);
        bUseHwDecoder = ptHwJpeg->fg_force_hw_mode;
    }
    else
    {
        ALOGD("{%s %d} fail.\n", __FUNCTION__, __LINE__);
        return -1;
    }
    
    ALOGD("2.1 image_width = %d, image_height = %d\n", cinfo->image_width,  cinfo->image_height);

    //make sure cts image can use sw decoder
    if (cinfo->out_color_space != JCS_YV16)
    {
        ALOGD("the colormode is not JCS_YV16, turn to sw decoder.\n");
        ptHwJpeg->fg_force_hw_mode = FALSE;
        return -1;
    }

    if (IS_HW_DECODE() && bUseHwDecoder && fgJpegIsFree())
    {
        if(!fgJpgMutexInit)
        {            
            pthread_mutex_init(&JpgLibMutex, NULL);
            ALOGD("pthread_mutex_init JpgLibMutex ....\n");
            fgJpgMutexInit = TRUE;
        }
        ret = pthread_mutex_trylock(&JpgLibMutex); 
        if(0 != ret)
        {
            ALOGD("pthread_mutex_trylock failed ret=%d!\n", ret);

            bUseHwDecoder = FALSE;
            ptHwJpeg->fg_force_hw_mode = bUseHwDecoder;
            
            return -1;
        }

        ALOGD("JpgLibMutex lock....\n");

        pthread_cond_init(&ptHwJpeg->JpgStopCond, NULL);
        pthread_mutex_init(&ptHwJpeg->JpgStopMutex, NULL);

        switch (cinfo->out_color_space) {
            case JCS_GRAYSCALE:
                cinfo->out_color_components = 1;
                break;
            case JCS_RGB:
                cinfo->out_color_components = 3;
                break;
            case JCS_RGB_565:
                cinfo->out_color_components = 3;
                break;
            case JCS_RGBA_8888:
                cinfo->out_color_components = 4;
                break;
            default:          /* else must be same colorspace as in file */
                cinfo->out_color_components = cinfo->num_components;
                break;
        }
        cinfo->output_components  = cinfo->out_color_components;

        if(cinfo->scale_denom && cinfo->out_color_space != JCS_YV16)
        {
            cinfo->output_width = cinfo->image_width / cinfo->scale_denom ;
            cinfo->output_height = cinfo->image_height / cinfo->scale_denom ;
        }

        switch(cinfo->out_color_space)
        {
            case JCS_RGBA_8888:
                ptHwJpeg->dest_bytes_per_pixel = 4;
                ptHwJpeg->src_bytes_per_pixel   = 4;
                break;
            case JCS_RGB:
                ptHwJpeg->dest_bytes_per_pixel = 3;            
                ptHwJpeg->src_bytes_per_pixel  = 4;            
                break;
            case JCS_RGB_565:
                ptHwJpeg->dest_bytes_per_pixel = 2;                        
                ptHwJpeg->src_bytes_per_pixel  = 2;            
                break;
            case JCS_YV16:
                ptHwJpeg->dest_bytes_per_pixel = 1;                        
                ptHwJpeg->src_bytes_per_pixel  = 1;
                break;
            default:
                break;
        }

        if (mtmplay_jpg_dec(cinfo))
        {
            cinfo->output_scanline = 0; 

            ALOGD("HW original image_width = %d, image_height = %d\n", cinfo->image_width,  cinfo->image_height);
            ALOGD("HW output_width = %d, output_height = %d\n", cinfo->output_width,  cinfo->output_height);
            pthread_mutex_unlock(&JpgLibMutex);

            return 0;
        }
        else
        {
            ALOGD("\n!!!-->HW Err.\n");

            ptHwJpeg->fg_force_hw_mode = FALSE;
            pthread_mutex_unlock(&JpgLibMutex);
            
            return -2;
        }
    }
    else
    {
        ptHwJpeg->fg_force_hw_mode = FALSE;
        ALOGD("{%s %d} fail.\n", __FUNCTION__, __LINE__);
        return -1;
    }
}

int jpg_readLine(j_decompress_ptr cinfo, JSAMPARRAY scanlines, JDIMENSION max_lines)
{
    JPEG_HW_DECODER_T* ptHwJpeg = NULL;
    boolean        bUseHwDecoder = FALSE;
    
    if (cinfo->client_data != NULL
        && *((INT32*)cinfo->client_data) == JPEG_HW_MAGIC_NUMBER)
    {
        ptHwJpeg = (JPEG_HW_DECODER_T*)(cinfo->client_data);
        bUseHwDecoder = ptHwJpeg->fg_force_hw_mode;
    }
    else
    {
        ALOGD("{%s %d} fail.\n", __FUNCTION__, __LINE__);
        return 0;
    }
    
    if (IS_HW_DECODE() && bUseHwDecoder)
    {
        JDIMENSION   output_pitch;
        UINT8        Jpg_DesBytePerPixel = ptHwJpeg->dest_bytes_per_pixel;
        UINT8        Jpg_SrcBytePerPixel = ptHwJpeg->src_bytes_per_pixel;

        int i, j, dst_offset = 0;
        UINT8* tempBuffer;

        if(!scanlines)
        {
            ALOGD("->ERR jpeg_read_scanlines  scanlines is null and return.\n");            
            return 0;
        }

        tempBuffer = (UINT8*)scanlines[0];
        if (!tempBuffer)
        {
            ALOGD("->ERR jpeg_read_scanlines  scanlines[0] is null and return\n");
            cinfo->output_scanline = cinfo->output_height;
            return 0;
        }

        if ((scanlines == NULL) || (tempBuffer == NULL))
        {
            ALOGD("scanlines==NULL\n");
            return 0;
        }

        output_pitch= (cinfo->output_width+0xf)&~0xf;

        if (cinfo->out_color_space == JCS_YV16)
        {
            dst_offset = output_pitch * cinfo->output_height;
        }
        if (cinfo->out_color_space == JCS_RGBA_8888)
        {
            memcpy(tempBuffer, (cinfo->input_iMCU_row + cinfo->output_scanline * output_pitch * Jpg_SrcBytePerPixel), cinfo->output_width*4);
        }
        else
        {

        //skip A
        for (i = 0;i < cinfo->output_width;i++)
        {
            switch(cinfo->out_color_space)
            {
                case JCS_RGBA_8888:
                    *(tempBuffer + i*Jpg_DesBytePerPixel) = *((UINT8*)cinfo->input_iMCU_row + cinfo->output_scanline * output_pitch * Jpg_SrcBytePerPixel + i * Jpg_SrcBytePerPixel);//R
                    *(tempBuffer + i*Jpg_DesBytePerPixel + 1) = *((UINT8*)cinfo->input_iMCU_row + cinfo->output_scanline * output_pitch * Jpg_SrcBytePerPixel + i * Jpg_SrcBytePerPixel + 1);//G
                    *(tempBuffer + i*Jpg_DesBytePerPixel + 2) = *((UINT8*)cinfo->input_iMCU_row + cinfo->output_scanline * output_pitch * Jpg_SrcBytePerPixel + i * Jpg_SrcBytePerPixel + 2);//B
                    *(tempBuffer + i*Jpg_DesBytePerPixel + 3) = *((UINT8*)cinfo->input_iMCU_row + cinfo->output_scanline * output_pitch * Jpg_SrcBytePerPixel + i * Jpg_SrcBytePerPixel+3 );//A
                    break;
                case JCS_RGB:
                    *(tempBuffer + i*Jpg_DesBytePerPixel) = *((UINT8*)cinfo->input_iMCU_row + cinfo->output_scanline * output_pitch * Jpg_SrcBytePerPixel + i * Jpg_SrcBytePerPixel);//R
                    *(tempBuffer + i*Jpg_DesBytePerPixel + 1) = *((UINT8*)cinfo->input_iMCU_row + cinfo->output_scanline * output_pitch * Jpg_SrcBytePerPixel + i * Jpg_SrcBytePerPixel + 1);//G
                    *(tempBuffer + i*Jpg_DesBytePerPixel + 2) = *((UINT8*)cinfo->input_iMCU_row + cinfo->output_scanline * output_pitch * Jpg_SrcBytePerPixel + i * Jpg_SrcBytePerPixel + 2);//B         
                    break;
                case JCS_RGB_565:
                    *(tempBuffer + i*Jpg_DesBytePerPixel+1) = *((UINT8*)cinfo->input_iMCU_row + cinfo->output_scanline * output_pitch * Jpg_SrcBytePerPixel + i * Jpg_SrcBytePerPixel+1); 
                    *(tempBuffer + i*Jpg_DesBytePerPixel) = *((UINT8*)cinfo->input_iMCU_row + cinfo->output_scanline * output_pitch * Jpg_SrcBytePerPixel + i * Jpg_SrcBytePerPixel);
                    break;
                case JCS_YV16:
                    *(tempBuffer + i*Jpg_DesBytePerPixel) = *((UINT8*)cinfo->input_iMCU_row + cinfo->output_scanline * output_pitch * Jpg_SrcBytePerPixel + i * Jpg_SrcBytePerPixel); 
                    if (i % 2 == 0)
                    {
                        *(tempBuffer + i*Jpg_DesBytePerPixel+dst_offset) = *((UINT8*)ptHwJpeg->CBuffer + cinfo->output_scanline * output_pitch * Jpg_SrcBytePerPixel + i * Jpg_SrcBytePerPixel ); 
                        *(tempBuffer + i*Jpg_DesBytePerPixel+dst_offset+1) = *((UINT8*)ptHwJpeg->CBuffer + cinfo->output_scanline * output_pitch * Jpg_SrcBytePerPixel + i * Jpg_SrcBytePerPixel +1); 
                    }
                    break;
                default:
                    break;
            }

        }
        }
        
        cinfo->output_scanline += max_lines;

        return 1;

    }
    else
    {
        return 0;
    }
}

boolean jpg_close(j_decompress_ptr cinfo)
{
    JPEG_HW_DECODER_T* ptHwJpeg = NULL;
    boolean        bUseHwDecoder   = FALSE;

    ALOGD("{%s %d} pid=%d.\n", __FUNCTION__, __LINE__, getpid());

    if (cinfo->client_data != NULL
        && *((INT32*)cinfo->client_data) == JPEG_HW_MAGIC_NUMBER)
    {
        ptHwJpeg = (JPEG_HW_DECODER_T*)cinfo->client_data;
        bUseHwDecoder = ptHwJpeg->fg_force_hw_mode;
    }
    else
    {
        ALOGD("{%s %d} fail.\n", __FUNCTION__, __LINE__);
        return FALSE;
    }

    if (IS_HW_DECODE() && bUseHwDecoder)
    {
        mtimage_jpg_dec_close(cinfo);
        
        return TRUE;
    }

    ALOGD("{%s %d} fail.\n", __FUNCTION__, __LINE__);
    return FALSE;
}


