#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include <math.h>
#include <string.h>
#include <pthread.h>
#include <assert.h>
#include <semaphore.h>
#include <time.h>
#include <include/core/SkBitmap.h>
#include <Bitmap.h>

#include <JNIHelp.h>

#include "drv_common.h"

#include <utils/Log.h>
#include <stdio.h>
#include "jni.h"

#include "mtimage.h"
#include "mtgfx.h"
#include "mtcommon.h"


#include "pictureapi.h"

/////add for resource manager////////
#ifdef PHOTO_SUPPORT_MTK_ANDROID_RM
#include "MtkResourceManager.h"
#include "include/RMClientPriority.h"
#endif

#include "mtvdec.h"
#include "properties.h"

typedef struct _PHOTO_ENV_T 
{
    JavaVM * vm;
    jobject thiz;
} PHOTO_ENV_T;

typedef struct _PHOTO_INSTANCE_T 
{
    bool                fgInit;
    int                 videoPath;
    #ifdef PHOTO_SUPPORT_MTK_ANDROID_RM
    IResourceComponent* vdpComp;
    #endif
}PHOTO_INSTANCE_T;



#ifdef ALL_PIC_VIDEOPATH

#ifdef PHOTO_SUPPORT_MTK_ANDROID_RM
void RmClose(IResourceComponent* VdpComp)
{
    ALOGD("Enter RmClose\r\n");
    // release component & resource when exit.
    if (VdpComp)
    {
        ALOGD("wait callback done before destory component.\n");
        if (VdpComp->CallbackDone(VdpComp) == FALSE)
        {
            return;
        }
        VdpComp->ReleaseAllResource(VdpComp);
        RMDestoryComponent(VdpComp);
        VdpComp = NULL;
    }
}

BOOL RmRelease(void* ctx, void* pv_data)
{
    ALOGD("Enter RmRelease\r\n");

    PHOTO_INSTANCE_T* ptPhoto;
    IResourceComponent* tmpComp = NULL;

    if (ctx != NULL)
    {
        ptPhoto = (PHOTO_INSTANCE_T*)ctx;
        tmpComp = ptPhoto->vdpComp;

        ALOGD("RmRelease ptPhoto=0x%x.\n", ptPhoto);
        if (ptPhoto->fgInit)
        {
            ALOGD("#### MTPHOTO4K_DeInit !!!");
            MTPHOTO4K_PauseFadeEffect();
            MTPHOTO4K_DeInit_Ex(ptPhoto->videoPath);
            ptPhoto->fgInit = FALSE;
        }
        
        tmpComp->ReleaseAllResource(tmpComp);
    }
    return TRUE;
}

BOOL RmOpen(int videoPath, PHOTO_INSTANCE_T* ptPhoto)
{
    IResourceComponent* tmpComp = NULL;

    ALOGD("Enter RmOpen.\n");
    if(ptPhoto == NULL)
    {
        return FALSE;
    }
        
    Resource    res = {0};
    int         ret, i;
    char        value[PROPERTY_VALUE_MAX];
    UINT32      u4VDecNum = 0;
    
    // create component;
    ret = RMCreateComponent("mtk4k2kphotoplayer", RmRelease, (void*)ptPhoto, &tmpComp );
    if (ret != 0)
    {
        ALOGE("RMCreateComponent fail, ret=%d.\n", ret);
        return FALSE;
    }

    ret = RMSetComponentPriority(tmpComp, RMClientPriority_PhotoVDP);
    if (ret != 0)
    {
        ALOGE("RMSetComponentPriority fail, ret=%d.\n", ret);
        RmClose(tmpComp);
        return FALSE;
    }
    
    // check resource either regist. if resource not regist, return error.
    if (RMCheckResource(VIDEOPATH))
    {
        //allocate the vdec resource to make sure video and photo can be mutex.
        property_get("rm.vdec.num", value, "0");
        u4VDecNum = atoi(value);
        if (u4VDecNum == 0) 
        {
            ret = MTVDEC_GetDecoderNum(&u4VDecNum);
            if (ret != MTR_OK)
            {
                ALOGE("MTVDEC_GetDecoderNum fail, ret=%d.\n", ret);
                RmClose(tmpComp);
                return FALSE;
            }
        }

        ALOGD("VIDEODECODER resource number %d.\n", u4VDecNum);

        for (i = 0; i < u4VDecNum; i++)
        {
            res.e_Type                  = RESOURCE_TYPE_NUMBERNIC;
            res.resourceName            = VIDEODECODER;
            res.u.NumbernicType.value   = i;

            if (!(tmpComp->AddResourceEx(tmpComp, &res)))
            {
                ALOGE("AddResourceEx VIDEODECODER fail.\n");
                RmClose(tmpComp);
                return FALSE;
            }
        }

        res.e_Type                  = RESOURCE_TYPE_NUMBERNIC;
        res.resourceName            = VIDEOPATH;
        res.u.NumbernicType.value   = videoPath;
        if (tmpComp->AddResourceEx(tmpComp, &res))
        {
            if (tmpComp->AllocResourceWithPriority(tmpComp, RMClientPriority_PhotoVDP))
            {
                ptPhoto->vdpComp = tmpComp;
                ALOGD(" RmOpen OK.\n");
                return TRUE;   
            }
            else
            {
                ALOGE("AllocResource fail.\n");
            }
        }
        else
        {
            ALOGE("AddResource fail.\n");
        }
    }
    else
    {
        ALOGE("Resource : %s not register\r\n", VIDEOPATH);
    }
    
    RmClose(tmpComp);   ////alloc resource failed, free all alloced resource by myself.
    
    return FALSE;
}
#endif
#endif

#define BITMAP_COLOR_FORMAT YUV_422_SEP

int error_code_switch(MT_RESULT_T ret)
{
    int error = PJR_NOT_OK;
    switch(ret)
    {
        case MTR_FBM_ERROR:
            error = PJR_FBM_ERROR;
            break;
        case MTR_OUT_OF_MEMORY:
            error = PJR_OUT_OF_MEMORY;
            break;
        case MTR_NOT_SUPPORTED:
            error = PJR_NOT_SUPPORTED;
            break;
        case MTR_PARAMETER_ERROR:
        case MTR_ERR_INV:
            error = PJR_INV_ARG;
            break;
        default:
            break;
    }

    return error;
}


int initPhotoPlay(bool b_init, int videoPath, int photoHandle)
{
#ifdef ALL_PIC_VIDEOPATH
    MT_RESULT_T     e_return = MTR_OK;
    int ret;
    PHOTO_INSTANCE_T*   ptPhoto = NULL;

    e_return = MTAL_InitExCb(TRUE);
    if (e_return != MTR_OK)
    {
        ALOGE("MTAL_InitExCb fail ret=%d.\n", e_return);
        return (b_init ? 0 : PJR_MTAL_INIT_ERROR);
    }

    if (b_init)
    {
        ptPhoto = (PHOTO_INSTANCE_T*)malloc(sizeof(PHOTO_INSTANCE_T));
        if (ptPhoto == NULL)
        {
            ALOGE("malloc fail.\n");
            return 0;
        }

        ALOGD("alloc ptPhoto=0x%x.\n", ptPhoto);
        
        memset(ptPhoto, 0, sizeof(PHOTO_INSTANCE_T));

        ptPhoto->videoPath = videoPath;

        #ifdef PHOTO_SUPPORT_MTK_ANDROID_RM
        if (!RmOpen(videoPath, ptPhoto))
        {
            ALOGE("#### RmOpen error!");
            free(ptPhoto);
            return 0;
        }
        #endif

        e_return = MTPHOTO4K_Init_Ex(videoPath);
        if (e_return != MTR_OK)
        {
            #ifdef PHOTO_SUPPORT_MTK_ANDROID_RM
            RmClose(ptPhoto->vdpComp);
            #endif
            ALOGE("#### MTPHOTO4K_Init error ret=%d!", e_return);
            ret = error_code_switch(e_return);
            free(ptPhoto);
            return 0;
        }
        else
        {
            ALOGD("#### MTPHOTO4K_Init ok!");
            ptPhoto->fgInit = TRUE;
            return (int)ptPhoto;
        }
    }
    else
    {
        if (photoHandle == 0)
        {
            ALOGE("Deinit handle is null.\n");
            return PJR_INV_ARG;
        }
        ptPhoto = (PHOTO_INSTANCE_T*)photoHandle;
        if (ptPhoto->fgInit)
        {
            MTPHOTO4K_PauseFadeEffect();
            MTPHOTO4K_DeInit_Ex(ptPhoto->videoPath);
            ptPhoto->fgInit = FALSE;
        }
        
        #ifdef PHOTO_SUPPORT_MTK_ANDROID_RM
        RmClose(ptPhoto->vdpComp);
        ptPhoto->vdpComp = NULL;
        #endif

        free(ptPhoto);

        return PJR_OK;
    }
#else
    return PJR_NOT_SUPPORTED;
#endif
}


static jint nativeRenderPhotoOnVdp(JNIEnv * env, jobject obj,
    jobject srcRect, jobject dstRect, jlong  srcYUVBitmap, jint rotation, jint photoHandle)
{
    #ifdef ALL_PIC_VIDEOPATH
    PHOTO_INSTANCE_T* ptPhoto;
    if (photoHandle == 0)
    {
        ALOGE("{%s} photoHandle is null.", __FUNCTION__);
        env->DeleteLocalRef(srcRect);
        env->DeleteLocalRef(dstRect);
        return PJR_INV_ARG;
    }
    ALOGD("{%s} srcYUVBitmap=%ld.", __FUNCTION__, srcYUVBitmap);
    ptPhoto = (PHOTO_INSTANCE_T*)photoHandle;
    if(ptPhoto->fgInit)
    {
        jclass         rectClass = env->FindClass("android/graphics/Rect");

        MTPHOTO4K_DISPLAY_INFO_EX_T     pic_info;
        int                         src_bottom, src_top, src_left, src_right, bottom, top, left, right;
        jfieldID                    field;
        MT_RESULT_T                 m_ret;
        MTIOMMU_PIN_RANGE_T         pin_range;
        MTGFX_CACHE_DATA            cache_data = {0};
        
        android::Bitmap* native_bitmap = (android::Bitmap*)srcYUVBitmap;
        SkBitmap* bitmap = new SkBitmap();
        native_bitmap->getSkBitmap(bitmap);
    
        ALOGD("colorType=%d.", bitmap->colorType());
        
        pic_info.eType = BITMAP_COLOR_FORMAT;//colorMode;
        bitmap->lockPixels();
        pic_info.u4DataAddr = (UINT32)bitmap->getAddr(0,0);
        if(pic_info.u4DataAddr == 0)
        {
            bitmap->unlockPixels();
            ALOGE(".....native EEEEEE render_photo_on_vdp  .....pic_info.u4DataAddr 0.... ");
            env->DeleteLocalRef(srcRect);
            env->DeleteLocalRef(dstRect);
            env->DeleteLocalRef(rectClass);
            delete bitmap;
            return PJR_INV_BITMAP;
        }
        pic_info.u4DataSize     = bitmap->getSize();
        pin_range.start         = pic_info.u4DataAddr;
        pin_range.size          = pic_info.u4DataSize;

        /*Here noticed: The color ARGB and ABGR is not same as java app definiation*/
        ALOGE(".....bitmap->colorType():%d,pic_info.u4DataAdd:0x%x,pic_info.u4DataSize:0x%x....\n",bitmap->colorType(),pic_info.u4DataAddr,pic_info.u4DataSize);
        if (bitmap->colorType() == kBGRA_8888_SkColorType)
        {
            pic_info.eType = RGB_TYPE_ABGR8888;
        }
        else if (bitmap->colorType() == kRGBA_8888_SkColorType)
        {
            pic_info.eType = RGB_TYPE_ARGB8888;
        }
        else
        {
            pic_info.eType = YUV_422_SEP;
        }
        
        if (pic_info.eType == YUV_422_SEP)
        {
            pic_info.u4DataPitch    = bitmap->rowBytes()/2;
        }
        else
        {
            pic_info.u4DataPitch    = bitmap->rowBytes();
        }
        pic_info.u4VideoPath    = ptPhoto->videoPath;

        field   = env->GetFieldID(rectClass, "bottom", "I");
        bottom  = env->GetIntField(dstRect, field);
        field   = env->GetFieldID(rectClass, "top", "I");
        top     = env->GetIntField(dstRect, field);
        field   = env->GetFieldID(rectClass, "left", "I");
        left    = env->GetIntField(dstRect, field);
        field   = env->GetFieldID(rectClass, "right", "I");
        right   = env->GetIntField(dstRect, field);

        if (left < 0 || right  <= left ||
            top < 0  || bottom <= top)
        {
            bitmap->unlockPixels();
            ALOGE(".....native EEEEEE render_photo_on_vdp  .....invalid dst rectangle.... ");
            env->DeleteLocalRef(srcRect);
            env->DeleteLocalRef(dstRect);
            env->DeleteLocalRef(rectClass);
            delete bitmap;
            return PJR_INV_ARG;
        }
        
        pic_info.rDispRegion.u4x        = left;
        pic_info.rDispRegion.u4y        = top;
        pic_info.rDispRegion.u4Width    = right - left;
        pic_info.rDispRegion.u4Height   = bottom - top;

        //special case handle
        if (pic_info.rDispRegion.u4Width == 1)
        {
            pic_info.rDispRegion.u4Width = 2;
        }

        field       = env->GetFieldID(rectClass, "bottom", "I");
        src_bottom  = env->GetIntField(srcRect, field);
        field       = env->GetFieldID(rectClass, "top", "I");
        src_top     = env->GetIntField(srcRect, field);
        field       = env->GetFieldID(rectClass, "left", "I");
        src_left    = env->GetIntField(srcRect, field);
        field       = env->GetFieldID(rectClass, "right", "I");
        src_right   = env->GetIntField(srcRect, field);

        env->DeleteLocalRef(srcRect);
        env->DeleteLocalRef(dstRect);
        env->DeleteLocalRef(rectClass);

        if (src_left < 0 || src_right > bitmap->width() || src_right <= src_left ||
            src_top < 0 || src_bottom > bitmap->height() || src_bottom <= src_top)
        {
            bitmap->unlockPixels();
            ALOGE(".....native EEEEEE render_photo_on_vdp  .....invalid src rectangle.... ");
            delete bitmap;
            return PJR_INV_ARG;
        }

        pic_info.rSrcRegion.u4x         = src_left;
        pic_info.rSrcRegion.u4y         = src_top;
        pic_info.rSrcRegion.u4Width     = src_right - src_left;
        pic_info.rSrcRegion.u4Height    = src_bottom - src_top;
        pic_info.eRotate = (MTPHOTO4K_ROTATION_T)rotation;

        ALOGE("....ping_rang start:0x%x, size:0x%x..... ",pin_range.start,pin_range.size);
        if (pic_info.eType == YUV_422_SEP)
        {
            if(mlock((VOID*)pin_range.start, pin_range.size) != 0)
            {
                ALOGE("[%s]mlock() errno = %d==\n",__FUNCTION__, errno);
                ALOGE("error msg: %s\n",strerror(errno));
                bitmap->unlockPixels();
                delete bitmap;
                return PJR_NOT_OK;
            }

            if(MTMMU_Pin_UserMem(&pin_range) != MTR_OK)
            {
                ALOGE("[%s]***MTMMU_Pin_UserMem Fail!\n", __FUNCTION__);
                munlock((VOID*)pin_range.start, pin_range.size);
                bitmap->unlockPixels();
                delete bitmap;
                return PJR_NOT_OK;
            }
        

            cache_data.u4_addr  = pin_range.start;
            cache_data.u4_size  = pin_range.size;
            cache_data.u4_fg    = MTGFX_FLUSH_DCACHE | MTGFX_INVALID_DCACHE;
            
            MTGFX_Flush_Dcache(&cache_data);
        }
        
        //ALOGD("{%s} bitmap address=0x%x, size=0x%x.\n", __FUNCTION__, pin_range.start, pin_range.size);
        m_ret = MTPHOTO4K_SetDisplayInfo_Ex(pic_info);
        
        if (pic_info.eType == YUV_422_SEP)
        {
            MTMMU_UnPin_UserMem(&pin_range);
            munlock((VOID*)pin_range.start, pin_range.size);
        }   
        
        if (m_ret != MTR_OK)
        {
            bitmap->unlockPixels();
            ALOGE("#### MTPHOTO4K_SetDisplayInfo error! ret=%d.\n",m_ret);
            delete bitmap;
            return error_code_switch(m_ret);
        }
        if (pic_info.eType != YUV_422_SEP)
        {
            m_ret = MTPHOTO4K_Show(1, TRUE);
            if(MTR_OK != m_ret)
            {
                bitmap->unlockPixels();
                ALOGE("#### MTPHOTO4K_Show error! ret=%d.\n",m_ret);
                delete bitmap;
                return error_code_switch(m_ret);
            }
        }
        bitmap->unlockPixels();
        delete bitmap;
        return PJR_OK;
    }   
    else
    {
        ALOGE(".....native EEEEEE render_photo_on_vdp  .....false.... ");
        env->DeleteLocalRef(srcRect);
        env->DeleteLocalRef(dstRect);
        return PJR_NOT_INIT;
    }
    #endif
    return PJR_NOT_OK;
}

#ifdef ALL_PIC_VIDEOPATH
void FadeCallBack(MTPHOTO4K_CALLBACK_TYPE_T eCallbackType, UINT32 u4Param, VOID* pv_tag)
{
    PHOTO_ENV_T*        info = (PHOTO_ENV_T*) pv_tag;
    JNIEnv*             env;
    jclass              photoClass;
    jmethodID           BuildEventMethod; 

    jint ret            = ((info->vm))->AttachCurrentThread((JNIEnv**)&env, NULL);
    photoClass          = env->GetObjectClass(info->thiz);
    BuildEventMethod    = env->GetMethodID(photoClass,"PhotoEvent","(I)V");  
    ALOGD("####  call PhotoEvent %d", eCallbackType);
    
    env->CallVoidMethod(info->thiz, BuildEventMethod, (int)eCallbackType);

    env->DeleteLocalRef(photoClass);
}
#endif

static jint nativeRegisterCallBackHandler(JNIEnv * env, jobject obj, jobject thiz)
{
#ifdef ALL_PIC_VIDEOPATH
    MT_RESULT_T     eRet;
    PHOTO_ENV_T*    envInfo = (PHOTO_ENV_T*)malloc(sizeof(PHOTO_ENV_T));
    jint            ret = env->GetJavaVM( &envInfo->vm);
    if(ret < 0)
    {
        ALOGE("{%s %d}GetJavaVM fail.\n", __FUNCTION__, __LINE__);
        return PJR_NOT_OK;
    }
 
    envInfo->thiz   = env->NewGlobalRef(thiz);
    env->DeleteLocalRef(thiz);

    eRet = MTPHOTO4K_RegCbFunc(FadeCallBack, (VOID*)envInfo);
    if (eRet != MTR_OK)
    {
        ALOGE("MTPHOTO4K_RegCbFunc fail, ret=%d.\n", eRet);
        return PJR_REG_CB_FAIL;
    }
#endif
    return PJR_OK;
}

static int nativePreparePhotoOnVdp(JNIEnv * env, jobject obj, 
    jobject srcRect, jobject dstRect, jlong  srcYUVBitmap, jint rotation, jint photoHandle)
{
#ifdef ALL_PIC_VIDEOPATH
    MTPHOTO4K_FADE_T    tFadeInfo = {0};
    jclass              rectClass = env->FindClass("android/graphics/Rect");

    MTPHOTO4K_DISPLAY_INFO_T    pic_info;
    int                         src_bottom, src_top, src_left, src_right, bottom, top, left, right;
    jfieldID                    field;
    MT_RESULT_T                 m_ret;
    MTIOMMU_PIN_RANGE_T         pin_range;
    MTGFX_CACHE_DATA            cache_data = {0};
    PHOTO_INSTANCE_T*           ptPhoto;

    android::Bitmap* native_bitmap = (android::Bitmap*)srcYUVBitmap;
    SkBitmap* bitmap = new SkBitmap();
    native_bitmap->getSkBitmap(bitmap);
    
    ALOGD("colorType=%d.", bitmap->colorType());
    
    pic_info.eType = BITMAP_COLOR_FORMAT;//colorMode;

    
    //SkBitmap* bitmap = (SkBitmap*)srcYUVBitmap;

    if (photoHandle == 0)
    {
        env->DeleteLocalRef(srcRect);
        env->DeleteLocalRef(dstRect);
        env->DeleteLocalRef(rectClass);
        delete bitmap;
        return PJR_INV_ARG;
    }

    ptPhoto = (PHOTO_INSTANCE_T*)photoHandle;

    if (!ptPhoto->fgInit)
    {
        env->DeleteLocalRef(srcRect);
        env->DeleteLocalRef(dstRect);
        env->DeleteLocalRef(rectClass);
        delete bitmap;
        ALOGE("{%s %d} fgInit is false.\n", __FUNCTION__, __LINE__);
        return PJR_NOT_INIT;
    }
    
    bitmap->lockPixels();
    tFadeInfo.photoData = (UINT32)bitmap->getAddr(0,0);
    if(tFadeInfo.photoData == 0)
    {
        bitmap->unlockPixels();
        env->DeleteLocalRef(srcRect);
        env->DeleteLocalRef(dstRect);
        env->DeleteLocalRef(rectClass);
        delete bitmap;
        ALOGE(".....nativePreparePhotoOnVdp  .....tFadeInfo.photoData 0.... ");
        return PJR_INV_BITMAP;
    }
    tFadeInfo.photoDataSize     = bitmap->getSize();
    tFadeInfo.photoDataPitch    = bitmap->rowBytes()/2;
    tFadeInfo.videoPath         = ptPhoto->videoPath;
    pin_range.start             = tFadeInfo.photoData;
    pin_range.size              = tFadeInfo.photoDataSize;

    field   = env->GetFieldID(rectClass, "bottom", "I");
    bottom  = env->GetIntField(dstRect, field);
    field   = env->GetFieldID(rectClass, "top", "I");
    top     = env->GetIntField(dstRect, field);
    field   = env->GetFieldID(rectClass, "left", "I");
    left    = env->GetIntField(dstRect, field);
    field   = env->GetFieldID(rectClass, "right", "I");
    right   = env->GetIntField(dstRect, field);

    if (left < 0 || right  <= left ||
        top < 0  || bottom <= top)
    {
        bitmap->unlockPixels();
        env->DeleteLocalRef(srcRect);
        env->DeleteLocalRef(dstRect);
        env->DeleteLocalRef(rectClass);
        delete bitmap;
        ALOGE(".....nativePreparePhotoOnVdp  .....invalid dst rectangle.... ");
        return PJR_INV_ARG;
    }
    
    tFadeInfo.dstRect.u4x       = left;
    tFadeInfo.dstRect.u4y       = top;
    tFadeInfo.dstRect.u4Width   = right - left;
    tFadeInfo.dstRect.u4Height  = bottom - top;

    //special case handle
    if (tFadeInfo.dstRect.u4Width == 1)
    {
        tFadeInfo.dstRect.u4Width = 2;
    }

    field       = env->GetFieldID(rectClass, "bottom", "I");
    src_bottom  = env->GetIntField(srcRect, field);
    field       = env->GetFieldID(rectClass, "top", "I");
    src_top     = env->GetIntField(srcRect, field);
    field       = env->GetFieldID(rectClass, "left", "I");
    src_left    = env->GetIntField(srcRect, field);
    field       = env->GetFieldID(rectClass, "right", "I");
    src_right   = env->GetIntField(srcRect, field);

    env->DeleteLocalRef(srcRect);
    env->DeleteLocalRef(dstRect);
    env->DeleteLocalRef(rectClass);

    if (src_left < 0 || src_right > bitmap->width() || src_right <= src_left ||
        src_top < 0 || src_bottom > bitmap->height() || src_bottom <= src_top)
    {
        bitmap->unlockPixels();
        delete bitmap;
        ALOGE(".....nativePreparePhotoOnVdp  .....invalid src rectangle.... ");
        return PJR_INV_ARG;
    }

    tFadeInfo.srcRect.u4x       = src_left;
    tFadeInfo.srcRect.u4y       = src_top;
    tFadeInfo.srcRect.u4Width   = src_right - src_left;
    tFadeInfo.srcRect.u4Height  = src_bottom - src_top;

    tFadeInfo.eRotate           = (MTPHOTO4K_ROTATION_T)rotation;
    if(mlock((VOID*)pin_range.start, pin_range.size) != 0)
    {
        ALOGE("[%s]mlock() errno = %d==\n",__FUNCTION__, errno);
        ALOGE("error msg: %s\n",strerror(errno));
        bitmap->unlockPixels();
        delete bitmap;
        return PJR_NOT_OK;
    }

    if(MTMMU_Pin_UserMem(&pin_range) != MTR_OK)
    {
        ALOGE("[%s]***MTMMU_Pin_UserMem Fail!\n", __FUNCTION__);
        munlock((VOID*)pin_range.start, pin_range.size);
        bitmap->unlockPixels();
        delete bitmap;
        return PJR_NOT_OK;
    }
    cache_data.u4_addr  = pin_range.start;
    cache_data.u4_size  = pin_range.size;
    cache_data.u4_fg    = MTGFX_FLUSH_DCACHE | MTGFX_INVALID_DCACHE;    
    MTGFX_Flush_Dcache(&cache_data);
    m_ret = MTPHOTO4K_PreparePhoto(tFadeInfo);
    MTMMU_UnPin_UserMem(&pin_range);
    munlock((VOID*)pin_range.start, pin_range.size);
    if (m_ret != MTR_OK)
    {
        ALOGE("MTPHOTO4K_PreparePhoto fail, ret=%d.\n", m_ret);
        bitmap->unlockPixels();
        delete bitmap;
        return error_code_switch(m_ret);
    }

    bitmap->unlockPixels();
    delete bitmap;
#endif
    return PJR_OK;
}

static int nativeStartFadeEffect(JNIEnv * env, jobject obj, 
    jint duration, jint bgColor, jint fadeType, jint photoHandle)
{
#ifdef ALL_PIC_VIDEOPATH
    MT_RESULT_T         m_ret;
    MTPHOTO4K_FADE_T    tFadeInfo = {0};
    PHOTO_INSTANCE_T*   ptPhoto;

    if (photoHandle == 0)
    {
        return PJR_INV_ARG;
    }

    ptPhoto = (PHOTO_INSTANCE_T*)photoHandle;

    if (!ptPhoto->fgInit)
    {
        return PJR_NOT_INIT;
    }

    tFadeInfo.videoPath     = ptPhoto->videoPath;
    tFadeInfo.duration      = duration;
    tFadeInfo.fadeType      = (fadeType == 1)? MTPHOTO4K_FADE_TYPE_OUT : MTPHOTO4K_FADE_TYPE_IN;
    tFadeInfo.fadeBgColor   = (bgColor  == 1) ? MTPHOTO4K_FADE_BGCOLOR_WHITE : MTPHOTO4K_FADE_BGCOLOR_BLACK;

    m_ret = MTPHOTO4K_StartFadeEffect(tFadeInfo);
    if (m_ret != MTR_OK)
    {
        ALOGE("MTPHOTO4K_StartFadeEffect fail, ret=%d.\n", m_ret);
        return error_code_switch(m_ret);
    }
#endif
    return PJR_OK;
}

static int nativePauseFadeEffect(JNIEnv * env, jobject obj, jint photoHandle)
{
    ALOGD("####  call nativePauseFadeEffect");
#ifdef ALL_PIC_VIDEOPATH
    return MTPHOTO4K_PauseFadeEffect();
#else
    return PJR_OK;
#endif
}

static int nativeRestartFadeEffect(JNIEnv * env, jobject obj, jint photoHandle)
{
    ALOGD("####  call nativeRestartFadeEffect");
#ifdef ALL_PIC_VIDEOPATH
    return MTPHOTO4K_RestartFadeEffect();
#else
    return PJR_OK;
#endif
}


#if 0
static int nativeStartPanoramaScroll(JNIEnv * env, jobject obj, 
    jobject srcRect, jobject dstRect, jint  srcYUVBitmap, jint videoPath, jint duration, jint direction, jint rotation)
{
#ifdef ALL_PIC_VIDEOPATH
    MTPHOTO4K_PANORAMA_T    tScrollInfo = {0};
    jclass                  rectClass = env->FindClass("android/graphics/Rect");

    int                     src_bottom, src_top, src_left, src_right, bottom, top, left, right;
    jfieldID                field;
    MT_RESULT_T             m_ret;
    
    SkBitmap* bitmap = (SkBitmap*)srcYUVBitmap;

    if (!fg_MTPHOTO4K_init)
    {
        ALOGE("{%s %d} fg_MTPHOTO4K_init is false.\n", __FUNCTION__, __LINE__);
        return PJR_NOT_INIT;
    }
    
    bitmap->lockPixels();
    tScrollInfo.photoData = (UINT32)bitmap->getAddr(0,0);
    if(tScrollInfo.photoData == 0)
    {
        bitmap->unlockPixels();
        ALOGE(".....nativeStartPanoramaScroll  .....tScrollInfo.photoData 0.... ");
        return PJR_INV_BITMAP;
    }
    tScrollInfo.photoDataSize   = bitmap->getSize();
    tScrollInfo.photoDataPitch  = bitmap->rowBytes()/2;
    tScrollInfo.videoPath       = videoPath;

    field   = env->GetFieldID(rectClass, "bottom", "I");
    bottom  = env->GetIntField(dstRect, field);
    field   = env->GetFieldID(rectClass, "top", "I");
    top     = env->GetIntField(dstRect, field);
    field   = env->GetFieldID(rectClass, "left", "I");
    left    = env->GetIntField(dstRect, field);
    field   = env->GetFieldID(rectClass, "right", "I");
    right   = env->GetIntField(dstRect, field);
    
    tScrollInfo.dstRect.u4x     = left;
    tScrollInfo.dstRect.u4y     = top;
    tScrollInfo.dstRect.u4Width = right - left;
    tScrollInfo.dstRect.u4Height= bottom - top;

    field       = env->GetFieldID(rectClass, "bottom", "I");
    src_bottom  = env->GetIntField(srcRect, field);
    field       = env->GetFieldID(rectClass, "top", "I");
    src_top     = env->GetIntField(srcRect, field);
    field       = env->GetFieldID(rectClass, "left", "I");
    src_left    = env->GetIntField(srcRect, field);
    field       = env->GetFieldID(rectClass, "right", "I");
    src_right   = env->GetIntField(srcRect, field);

    if (src_left < 0 || src_right > bitmap->width() || src_right < src_left ||
        src_top < 0 || src_bottom > bitmap->height() || src_bottom < src_top)
    {
        bitmap->unlockPixels();
        ALOGE(".....native EEEEEE render_photo_on_vdp  .....invalid src rectangle.... ");
        return PJR_INV_ARG;
    }

    tScrollInfo.srcRect.u4x     = src_left;
    tScrollInfo.srcRect.u4y     = src_top;
    tScrollInfo.srcRect.u4Width = src_right - src_left;
    tScrollInfo.srcRect.u4Height= src_bottom - src_top;

    tScrollInfo.duration    = duration;
    tScrollInfo.direction   = (MTPHOTO4K_SCROLL_DIRECTION)direction;
    tScrollInfo.eRotate     = (MTPHOTO4K_ROTATION_T)rotation;
    
    m_ret = MTPHOTO4K_StartPanoramaScroll(tScrollInfo);
    if (m_ret != MTR_OK)
    {
        ALOGE("MTPHOTO4K_StartPanoramaScroll fail, ret=%d.\n", m_ret);
        bitmap->unlockPixels();
        return error_code_switch(m_ret);
    }
    bitmap->unlockPixels();
#endif
    return PJR_OK;
}

static int nativePausePanoramaScroll(JNIEnv * env, jobject obj)
{
    ALOGD("####  call nativePausePanoramaScroll");
#ifdef ALL_PIC_VIDEOPATH
    return MTPHOTO4K_PausePanoramaScroll();
#else
    return PJR_OK;
#endif
}

static int nativeRestartPanoramaScroll(JNIEnv * env, jobject obj)
{
    ALOGD("####  call nativePausePanoramaScroll");
#ifdef ALL_PIC_VIDEOPATH
    return MTPHOTO4K_RestartPanoramaScroll();
#else
    return PJR_OK;
#endif
}
#endif
static bool nativeIs4KPanel(JNIEnv * env, jobject obj)
{
    ALOGD("####  call nativeIs4KPanel");
#ifdef ALL_PIC_VIDEOPATH
    BOOL        bIs4K;
    MT_RESULT_T mRet;
    mRet = MTPHOTO4K_Is4KPanel(&bIs4K);
    if (mRet != MTR_OK)
    {
        ALOGE("####MTPHOTO4K_Is4KPanel return error:%d.", mRet);
        return FALSE;
    }
    else
    {
        return bIs4K;
    }
#else
    return FALSE;
#endif
}


/**
*Argument: b_init 
*false: do deinitial function after exit photo playing. true: do initial function before enter photo playing
**/
static jint nativeInitPhotoPlay(JNIEnv * env, jobject obj, jint videoPath)
{
    ALOGD(".....nativeInitPhotoPlay:..... videoPath=%d ", videoPath);
    return initPhotoPlay(TRUE, videoPath, 0);
}

static jint nativeDeinitPhotoPlay(JNIEnv * env, jobject obj, jint photoHandle)
{
    ALOGD(".....nativeDeInitPhotoPlay:..... photoHandle= 0x%x", photoHandle);
    return initPhotoPlay(FALSE, 0, photoHandle);
}


static const char *classPathName = "com/mediatek/jni/PhotoRender";


static JNINativeMethod methods[] = {
  {"nativeInitPhotoPlay",       "(I)I",                                                 (void*)nativeInitPhotoPlay},
  {"nativeDeinitPhotoPlay",     "(I)I",                                                 (void*)nativeDeinitPhotoPlay},
  {"nativeRenderPhotoOnVdp",    "(Landroid/graphics/Rect;Landroid/graphics/Rect;JII)I", (void*)nativeRenderPhotoOnVdp},
  {"nativeRegCbHandler",        "(Lcom/mediatek/jni/PhotoRender;)I",                    (void*)nativeRegisterCallBackHandler},
  {"nativePreparePhotoOnVdp",   "(Landroid/graphics/Rect;Landroid/graphics/Rect;JII)I", (void*)nativePreparePhotoOnVdp},
  {"nativeStartFade",           "(IIII)I",                                              (void*)nativeStartFadeEffect},
  {"nativePauseFade",           "(I)I",                                                 (void*)nativePauseFadeEffect},
  {"nativeRestartFade",         "(I)I",                                                 (void*)nativeRestartFadeEffect},
  {"nativeIs4KPanel",           "()Z",                                                  (void*)nativeIs4KPanel},
    
};

/*
 * Register several native methods for one class.
 */
static int registerNativeMethods(JNIEnv* env, const char* className,
    JNINativeMethod* gMethods, int numMethods)
{
    jclass clazz;

    clazz = env->FindClass(className);
    if (clazz == NULL) {
        ALOGE("Native registration unable to find class '%s'", className);
        return JNI_FALSE;
    }
    if (env->RegisterNatives(clazz, gMethods, numMethods) < 0) {
        ALOGE("RegisterNatives failed for '%s'", className);
        return JNI_FALSE;
    }

    return JNI_TRUE;
}

/*
 * Register native methods for all classes we know about.
 *
 * returns JNI_TRUE on success.
 */
static int registerNatives(JNIEnv* env)
{
  if (!registerNativeMethods(env, classPathName, methods, sizeof(methods) / sizeof(methods[0]))) {
    return JNI_FALSE;
  }

  return JNI_TRUE;
}

// ----------------------------------------------------------------------------

/*
 * This is called by the VM when the shared library is first loaded.
 */
 
typedef union {
    JNIEnv* env;
    void* venv;
} UnionJNIEnvToVoid;

jint JNI_OnLoad(JavaVM* vm, void* reserved)
{
    UnionJNIEnvToVoid uenv;
    uenv.venv = NULL;
    jint result = -1;
    JNIEnv* env = NULL;
    
    ALOGI("JNI_OnLoad");

    if (vm->GetEnv(&uenv.venv, JNI_VERSION_1_4) != JNI_OK) {
        ALOGE("ERROR: GetEnv failed");
        goto bail;
    }
    env = uenv.env;

    if (registerNatives(env) != JNI_TRUE) {
        ALOGE("ERROR: registerNatives failed");
        goto bail;
    }
    
    result = JNI_VERSION_1_4;
    
bail:
    return result;
}

