/**********
This library is free software; you can redistribute it and/or modify it under
the terms of the GNU Lesser General Public License as published by the
Free Software Foundation; either version 2.1 of the License, or (at your
option) any later version. (See <http://www.gnu.org/copyleft/lesser.html>.)

This library is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
more details.

You should have received a copy of the GNU Lesser General Public License
along with this library; if not, write to the Free Software Foundation, Inc.,
51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
**********/
// "liveMedia"
// Copyright (c) 1996-2011 Live Networks, Inc.  All rights reserved.
// A simple UDP source, where every UDP payload is a complete frame
// Implementation

#include "BasicUDPSource.hh"
#include <GroupsockHelper.hh>
#include <utils/Log.h>
#include <cutils/log.h>
#include "mtk_log_ext.h"

#define USER_DELEY 150

#define  USHORT_MAX 65535

#define LIVEMEDIA_DBG( x...) \
    do { \               
		    LOGD("[LIVEMEDIA] " x); \
    } while(0)
    

bool fHavepacketlost = False;
BasicUDPSource* BasicUDPSource::createNew(UsageEnvironment& env,
					Groupsock* inputGS) {
  return new BasicUDPSource(env, inputGS);
}

BasicUDPSource::BasicUDPSource(UsageEnvironment& env, Groupsock* inputGS)
  : FramedSource(env), fInputGS(inputGS), fHaveStartedReading(False) ,fHaveSeenFirstPacket(False){
  // Try to use a large receive buffer (in the OS):
  increaseReceiveBufferTo(env, inputGS->socketNum(), 50*1024);

  // Make the socket non-blocking, even though it will be read from only asynchronously, when packets arrive.
  // The reason for this is that, in some OSs, reads on a blocking socket can (allegedly) sometimes block,
  // even if the socket was previously reported (e.g., by "select()") as having data available.
  // (This can supposedly happen if the UDP checksum fails, for example.)
  makeSocketNonBlocking(fInputGS->socketNum());
}

BasicUDPSource::~BasicUDPSource(){
  envir().taskScheduler().turnOffBackgroundReadHandling(fInputGS->socketNum());
}

void BasicUDPSource::doGetNextFrame() {
  if (!fHaveStartedReading) {
    // Await incoming packets:
    envir().taskScheduler().turnOnBackgroundReadHandling(fInputGS->socketNum(),
	 (TaskScheduler::BackgroundHandlerProc*)&incomingPacketHandler, this);
    fHaveStartedReading = True;
  }
}

void BasicUDPSource::doStopGettingFrames() {
  envir().taskScheduler().turnOffBackgroundReadHandling(fInputGS->socketNum());
  fHaveStartedReading = False;
}


void BasicUDPSource::incomingPacketHandler(BasicUDPSource* source, int /*mask*/){
  source->incomingPacketHandler1();
}

void BasicUDPSource::incomingPacketHandler1() {
  if (!isCurrentlyAwaitingData()) return; // we're not ready for the data yet

  // Read the packet into our desired destination:
  struct sockaddr_in fromAddress;
#ifndef NEED_BUFFER_POOL
  unsigned char arHeader[RTP_HEADER_LEN] = {0,};
  unsigned char* fTmpBuffer = fTo;
  struct timeval	local_time_print;
  fFrameSize = fReadCount = 0;
  
  static unsigned short dataNo = 0 ;
  
	do
	{
		if (!fInputGS->handleRead(fTmpBuffer, fMaxSize - fFrameSize, fReadSize, fromAddress))
			return;

		//printf("fReadSize = %u\n", fReadSize);

		if (fReadCount == 0 && fReadSize == 0)
			return;

		if (fReadSize == 0)
		{
			//printf("read 0, fReadCount = %u, fFrameSize = %u\n", fReadCount, fFrameSize);
			break;
		}
		
		unsigned short fSeqNo = ntohs(*(uint16_t*)(fTmpBuffer + 2));
		
		#ifdef NEED_BUFFER_POOL	
			unsigned long fTimeStamp32 = ntohl(*(uint32_t*)(fReadBuffer + 4));
		#else
			unsigned long fTimeStamp32 = ntohl(*(uint32_t*)(fTmpBuffer + 4));
		#endif	
		//struct timeval	local_time_print;
		gettimeofday(&local_time_print,NULL);		
		double stardard_time= (double)USER_DELEY/1000.0;
		
		// reset pakcet lost flag before use
		fHavepacketlost = False;

		if (!fHaveSeenFirstPacket)
	    {
			fNextExpectedSeqNo = fSeqNo + 1;
			fHaveSeenFirstPacket = true;
			LIVEMEDIA_DBG("RTP Recv First Packet. SeqNo = %u\n",fSeqNo);
						  
	    }
		else
		{
			if (fNextExpectedSeqNo != fSeqNo)
			{
				int diff = (fSeqNo - fNextExpectedSeqNo + USHORT_MAX - 1) % (USHORT_MAX -1);
				LIVEMEDIA_DBG("RTP Recv SeqNo = %u, Expect SeqNo = %u, Packet lost = %d\n",
						  fSeqNo, fNextExpectedSeqNo, diff);
				fHavepacketlost = True;
			}
			fNextExpectedSeqNo = fSeqNo + 1;
		}
		*(rtp_header_len_t*)fTmpBuffer = *(rtp_header_len_t*)arHeader;
		fTmpBuffer += (fReadSize - RTP_HEADER_LEN);
		fFrameSize += (fReadSize - RTP_HEADER_LEN);
		*(rtp_header_len_t*)arHeader = *(rtp_header_len_t*)fTmpBuffer;

		if((dataNo++ % 1000) == 0)
		{
		    LIVEMEDIA_DBG( "RTP....Recv data..SeqNo = %d .Time:(%lu.%03lu)s.\n",
				fSeqNo,local_time_print.tv_sec, local_time_print.tv_usec/1000);
			
		}
		
		if (++fReadCount > MAX_READ_COUNT)
		{
			//printf("read 10ts, fReadCount = %u, fFrameSize = %u\n", fReadCount, fFrameSize);
			break;
		}
		
	}while(1);
#else
  if (!fInputGS->handleRead(fTo, fMaxSize, fFrameSize, fromAddress)) return;
  printf("BasicUDPSource....read 1ts.....\n");
#endif
  // Tell our client that we have new data:
  afterGetting(this); // we're preceded by a net read; no infinite recursion
}
