#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <ctype.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>

#include <stdarg.h>
#include <fcntl.h>

#include <errno.h>

void die(const char *why, ...)
{
    va_list ap;

    va_start(ap, why);
    fprintf(stderr,"error: ");
    vfprintf(stderr, why, ap);
    fprintf(stderr,"\n");
    va_end(ap);
    exit(1);
}

void do_mknod(char* device_list)
{
    char line[1024];
    FILE* f = NULL;

    f = fopen(device_list, "r");

    if (NULL == f)
    {
        fprintf(stderr, "failed to open file %s\n", device_list);
        return;
    }

    while (fgets(line, sizeof(line), f) != NULL)
    {
        char* device_name = NULL;
        char* type = NULL;
        char* tmp = NULL;
        mode_t permission = 0;
        int uid = 0, gid = 0;
        int major = 0, minor = 0;
        int res;
        errno = 0;

        if (!line[0])
        {
            break;
        }

        tmp = strtok(line, " \n");
        if (NULL == tmp) {
            continue;
        }
        device_name = strdup(tmp);

        if (NULL == device_name)
        {
            break;
        }

        tmp = strtok(NULL, " \n");
        if (NULL == tmp) {
        	free(device_name);
            continue;
        }
        type = strdup(tmp);
        if (NULL == type)
        {
            free(device_name);
            break;
        }

        tmp = strtok(NULL, " \n");
        if (NULL == tmp) {
            free(device_name);
            free(type);
            continue;
        }
        permission = strtol(tmp, NULL, 8);

        tmp = strtok(NULL, " \n");
        if (NULL == tmp) {
            free(device_name);
            free(type);
            continue;
        }
        uid = atoi(tmp);

        tmp = strtok(NULL, " \n");
        if (NULL == tmp) {
            free(device_name);
            free(type);
            continue;
        }
        gid = atoi(tmp);

        if (0 == strcmp("d", type))
        {
            //mode = (S_IFDIR | permission);

            res = mkdir(device_name, S_IFDIR);

            if (res == -1)
            {
                fprintf(stderr, "failed to mkdir %s, error: %s\n", device_name, strerror(errno));
                free(device_name);
                free(type);
                continue;
            }
        }
        else
        {
            tmp = strtok(NULL, " \n");
            if (NULL == tmp) {
                free(device_name);
                free(type);
                continue;
            }
            major = atoi(tmp);

            tmp = strtok(NULL, " \n");
            if (NULL == tmp) {
                free(device_name);
                free(type);
                continue;
            }
            minor = atoi(tmp);

            if (0 == strcmp("b", type))
            {
                 //mode = (S_IFBLK | permission);
                 res = mknod(device_name, S_IFBLK, makedev(major, minor));

                 if (res == -1)
                 {
                     fprintf(stderr, "failed to mknod %s, error: %s\n", device_name, strerror(errno));
                     free(device_name);
                     free(type);
                     continue;
                 }
            }
            else if (0 == strcmp("c", type))
            {
                //mode = (S_IFCHR | permission);
                res = mknod(device_name, S_IFCHR, makedev(major, minor));

                if (res == -1)
                {
                    fprintf(stderr, "failed to mknod %s, error: %s\n", device_name, strerror(errno));
                    free(device_name);
                    free(type);
                    continue;
                }
            }
            else
            {
                // do nothing
                free(device_name);
                free(type);
                continue;
            }
        }
        chmod(device_name, permission);
        chown(device_name, uid, gid);
        free(device_name);
        free(type);
        //printf("%s %s %d %d %d %d %d\n", device_name, type, mode, uid, gid, major, minor);
    }

    fclose(f);
}

int main(int argc, char *argv[])
{
    int i;

    if (1 >= argc)
    {
        die("Usage: %s <dev.txt #1> <dev.txt #2> <dev.txt #3> ...\n", argv[0]);
    }
    for (i = 1 ; i < argc ; i++)
    {
        do_mknod(argv[i]);
    }

    return 0;
}
