/*
 * Copyright (C) 2012 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#include <errno.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#define LOG_NDEBUG 0
#define LOG_TAG "MTK PowerHAL"
#include <utils/Log.h>
#include <cutils/properties.h>

#include <hardware/hardware.h>
#include <hardware/power.h>
#include <net/if.h>

#include <stdio.h>
#include <stdlib.h>
#include <netdb.h>

#include <linux/sockios.h>
#include <sys/ioctl.h>
#include <netinet/in.h>
#include "u_common.h"
#include "app_util/mtktvapi/mtktvapi_config_type_custom.h"
#include "app_util/mtktvapi/a_mtktvapi_config.h"
#include "cutils/properties.h"



static const char WOPACKET_PROP_NAME[]    = "mtk.wopacket.force_default_on";


#define PROTOCOL_TCP	6
#define PROTOCOL_UDP	17


struct ioctl_wop_para_cmd
{
    UCHAR protocol_type;
    UCHAR port_count;
    UINT32 *port_array;
};

/*
*This API reduce  multi blank into one
*/
static char * _remove_multi_blank(char * src, char * desc)
{
    char * ptr = src;
    while(*ptr == ' ') ptr++;
    src = ptr;
    ptr = desc;
    while(*src != '\0')
    {
        if(*src != 0x20 || (*src == 0x20 && *(src +1) != 0x20))
        {
            *desc++ = *src++;            
        }else
        {
            src++;
        }
    }
    *desc = '\0';
    return ptr;
        
}


/*
*This API used to get the target string which speify by the index.
* For example:  there is an  array , a[0],a[1]..., if the index is 1, it will copy a[1] to s_result.
*
*/
static void _parseLine(char *s_line, char ss_result[][64], unsigned int l,unsigned int r)
{
    unsigned int i        = 0;
    char * ps_item = NULL;
    char * delim   = " ";
    char * ps_temp;
    char desc[256] = {0};
    s_line[strlen(s_line)-1]='\0';
    ps_temp = _remove_multi_blank(s_line, desc);
    ps_item = strtok(ps_temp, delim);
    while(ps_item != NULL)
    {   
        if(i >= l)break;
        if(r < strlen(ps_item))break;
        strncpy(ss_result[i], ps_item ,64);
        ps_item = strtok(NULL, delim);   
        i++;
    } 
    
}

static int _parseRemotePort(char * rem_address)
{
    char * ps_temp = strstr(rem_address, ":");
    if(ps_temp == NULL) return -1;
    ps_temp++; // ps_temp = :port, after ++ , ps_temp = port
    int x = 0;
    sscanf(ps_temp, "%x", &x);
    return x;
    
}


static int _getlocalIp(char * ifname, char * ip)
{      
    int s = 0, addr = 0;
    struct ifreq ifr;    
    memset(&ifr, 0, sizeof(struct ifreq));
    strncpy(ifr.ifr_name, ifname, IFNAMSIZ);
    ifr.ifr_name[IFNAMSIZ-1] = 0;
    if((s = socket(AF_INET, SOCK_DGRAM, 0)) < 0)
    {
        return -1;
    }
    if (ioctl(s, SIOCGIFADDR, &ifr) < 0)
    {
        return -1;
    } else
    {
        addr = ((struct sockaddr_in *)&ifr.ifr_addr)->sin_addr.s_addr;
    }
    snprintf(ip,20, "%d.%d.%d.%d",
            addr & 0xff,
            ((addr >> 8) & 0xff),
            ((addr >> 16) & 0xff),
            ((addr >> 24) & 0xff));

    
    return 0;
}

static int _nw_net_get_port(const char * s_file, int * pi4_ports, int i4_max)
{
    FILE  * fp             = NULL; 
    char   s_line[256]     = {0};
    fp = fopen(s_file, "rt");

    if (fp == NULL)
    {
        //c_dbg_stmt("[NW]Cannot open %s:%s\n", s_file, strerror(errno));
        return -1;
    }    
    int i = 0;
    while(fgets(s_line, 255, fp) != NULL)
    {
        /*s[0] : sl
                *s[1] : local_address
                *s[2] : rem_address
                *s[3] : st
                *s[4] : tx_queue
                *s[5] : rx_queut
                *s[6] : tr
                *s[7] : tm_when
                *s[8] : retrnsnt
                *s[9] : uid
                *s[10] : timeout
                *s[11] : innod
                *s[12] :
                */       
        char s[12][64] = {{0}};
        _parseLine(s_line, s, 12, 64);
      	//  c_dbg_stmt("Test: Status = %s :", s[3]);
       	if(strncmp(s[3], "01", 2) != 0) continue; // 01 is the status : ESTABLISHED
		//if(_compSrcIP("www.mtalk.google.com",s[2]) != 0) continue;//src ip must equal remote IP
		
        int i4_port = _parseRemotePort(s[2]);//remote port
		int i4_localport = 0;
        if(i4_port > 0)
        {
            //It is valid port. save it: Todo
            if(i >= i4_max)break;
         //   c_dbg_stmt("port = %d\n", i4_port);
         	if((i4_port == 5228) || (i4_port == 5229) || (i4_port == 5230) || (i4_port == 443))
         		{
					i4_localport = _parseRemotePort(s[1]);
					if(i4_localport > 0)
					{
						pi4_ports[i++] = i4_localport;
						ALOGE("XXXXXXXXget valid v4 local port %d \n",i4_localport);
					}
         		}
        }
    }
   fclose(fp);
   return 0;
}
static int _nw_net_get_port_ex(const char * s_file, int * pi4_v4ports,int * pi4_v6ports,
	int i4_max)
{
    FILE  * fp             = NULL; 
    char   s_line[256]     = {0};
	int i = 0;
    fp = fopen(s_file, "rt");

    if (fp == NULL)
    {
        //c_dbg_stmt("[NW]Cannot open %s:%s\n", s_file, strerror(errno));
        return -1;
    }    
    
    while(fgets(s_line, 255, fp) != NULL)
    {
        /*s[0] : sl
                *s[1] : local_address
                *s[2] : rem_address
                *s[3] : st
                *s[4] : tx_queue
                *s[5] : rx_queut
                *s[6] : tr
                *s[7] : tm_when
                *s[8] : retrnsnt
                *s[9] : uid
                *s[10] : timeout
                *s[11] : innod
                *s[12] :
                */       
        char s[12][64] = {{0}};
        _parseLine(s_line, s, 12, 64);
      	//  c_dbg_stmt("Test: Status = %s :", s[3]);
       	if(strncmp(s[3], "01", 2) != 0) continue; // 01 is the status : ESTABLISHED
		//if(_compSrcIP("www.mtalk.google.com",s[2]) != 0) continue;//src ip must equal remote IP
		
        int i4_port = _parseRemotePort(s[2]);
		int i4_localport = 0;
        if(i4_port > 0)
        {
            //It is valid port. save it: Todo
            if(i >= i4_max)break;
         //   c_dbg_stmt("port = %d\n", i4_port);
         	if((i4_port == 5228) || (i4_port == 5229) || (i4_port == 5230) || (i4_port == 443))
         	{
				if(strncmp(s[2],"00000000000000000000FFFF",24) == 0)
				{
					i4_localport = _parseRemotePort(s[1]);
					if(i4_localport > 0)
					{
						ALOGE("XXXXXXXXget valid v4port %d \n",i4_localport);
						pi4_v4ports[i++] = i4_localport;
					}
				}
				else if(strncmp(s[2],"0000000000000000FFFF",20) == 0)
				{
					i4_localport = _parseRemotePort(s[1]);
					if(i4_localport > 0)
					{
						ALOGE("XXXXXXXXget valid Ex v4port %d \n",i4_localport);
						pi4_v4ports[i++] = i4_localport;
					}
				}
				else
				{
					i4_localport = _parseRemotePort(s[1]);
					if(i4_localport > 0)
					{
						ALOGE("XXXXXXXXget valid v6port %d \n",i4_localport);
						pi4_v6ports[i++] = i4_localport;
					}
				}
         	}
        }
    }
   fclose(fp);
   return 0;
}

int a_nw_wlan_set_ip_and_port_to_driver()
{
	int i4_ret = 0;
	int n = 0;
	int i = 0;
	int j = 0;
	int k = 0;
	#define MAX_ARG_LEN 512
	#define MAX_NUM (20)
	char  s_local_ip[20]		= {0};
	int a_tcp_port[MAX_NUM + 1] = {0};
	int a_udp_port[MAX_NUM + 1] = {0};
	int a_tcp6_port[MAX_NUM + 1] = {0};
	int a_tcp6_port2[MAX_NUM + 1] = {0};
	int a_udp6_port[MAX_NUM + 1] = {0};
	const char s_tcp_file_path[] = "/proc/net/tcp";
	const char s_udp_file_path[] = "/proc/net/udp";
	const char s_tcp6_file_path[] = "/proc/net/tcp6";
	const char s_udp6_file_path[] = "/proc/net/udp6";
	char s_tmp[MAX_ARG_LEN]={0};

	////for ethernet wopacket
	
	unsigned int eth_tcp_port[MAX_NUM + 1] = {0};
	unsigned int tcp_count_idx = 0;
	unsigned int eth_udp_port[MAX_NUM + 1] = {0};
	unsigned int udp_count_idx = 0;
	struct ioctl_wop_para_cmd eth_wopacket_para;

	////
	
	ALOGD("XXXXXenter: AP a_nw_wlan_set_ip_and_port_to_driver\n");
	
	ALOGD("XXXXX AP first clear all ethernet port\n");
	i4_ret = ifc_clear_Ethernet_listen_port("eth0",NULL);
	if(i4_ret < 0)
	{
		ALOGE("XXXXX ethernet wop failed to clear port");
	}
		
	/*set tcp ip*/
	char *argv[4]={"iwpriv","wlan0","set",0};
	argv[3] = "wow_ip=0.0.0.0";
	i4_ret = iwpriv_main(4,argv);//set ip to any
	if(i4_ret < 0)
	{
		ALOGE("XXXXXset tcp ip error");
	}
	/*set tcp port*/
	n = 0;
	i4_ret = _nw_net_get_port(s_tcp_file_path, a_tcp_port, MAX_NUM-3);
	if(i4_ret != 0)
	{	 
		return -1;
	}
	snprintf(s_tmp,512, "%s", "wow_tcp_port_v4=8008,8009,1900,");
	/* eth wop  */
	eth_tcp_port[0] = 8008;
	eth_tcp_port[1] = 8009;
	eth_tcp_port[2] = 1900;
	tcp_count_idx += 3;
	
	for(i=0; i<MAX_NUM-3;i++)
	{
		if(a_tcp_port[i] > 0)
		{
			char s[10]={0};
			snprintf(s,10, "%d,", a_tcp_port[i]);
			eth_tcp_port[tcp_count_idx++] = a_tcp_port[i];
			strncat(s_tmp, s,10);
			n ++;			 
		}
	}


		
	/*set tcp v6  port*/
	
	i4_ret = _nw_net_get_port_ex(s_tcp6_file_path, a_tcp6_port2,a_tcp6_port, MAX_NUM);
	if(i4_ret != 0)
	{	 
		return -1;
	}

	//////////////
	for(i=0; i<MAX_NUM;i++)
	{
		if(a_tcp6_port2[i] > 0)
		{
			char s[10] = {0};
			snprintf(s,10 ,"%d,", a_tcp6_port2[i]);
			eth_tcp_port[tcp_count_idx++] = a_tcp6_port2[i];
			strncat(s_tmp, s ,10);
			n ++;			 
		}
	}
	//if(n>0)
	//{
		s_tmp[strlen(s_tmp) -1] ='\0'; //remove last ','	  
		ALOGD("(WOWL)tcp v6 port2 = %s", s_tmp);
		argv[3] = s_tmp;
		i4_ret = iwpriv_main(4,argv);
		if(i4_ret < 0)
		{
			ALOGE("XXXXXset tcp v6 port 2 error");
		}
	//}
	n = 0;
	memset(s_tmp, 0, 512);
	snprintf(s_tmp,512 ,"%s", "wow_tcp_port_v6=");
	for(i=0; i<MAX_NUM;i++)
	{
		if(a_tcp6_port[i] > 0)
		{
			char s[10] = {0};
			snprintf(s, 10 ,"%d,", a_tcp6_port[i]);
			eth_tcp_port[tcp_count_idx++] = a_tcp6_port[i];
			strncat(s_tmp, s ,10);
			n ++;			 
		}
	}
	if(n>0)
	{
		s_tmp[strlen(s_tmp) - 1] ='\0'; //remove last ','		 
		ALOGD("(WOWL)tcp v6 port= %s", s_tmp);
		argv[3] = s_tmp;
		i4_ret = iwpriv_main(4,argv);
		if(i4_ret < 0)
		{
			ALOGE("XXXXXset tcp v6 port error");
		}
	}

	eth_wopacket_para.protocol_type = PROTOCOL_TCP;
	eth_wopacket_para.port_count = tcp_count_idx;
	eth_wopacket_para.port_array = eth_tcp_port;
	
	for(j =0;j < tcp_count_idx;j++)
	{
		ALOGE("XXXXXethernet tcp port%d is %d",j,eth_wopacket_para.port_array[j]);
	}
	ifc_set_Ethernet_listen_port("eth0",&eth_wopacket_para);
	if(i4_ret < 0)
	{
		ALOGE("XXXXXethernet wop failed to set tcp_v6 port");
	}
		
	//set udp default port 5353 
	argv[3] = "wow_udp_port_v4=5353";
	eth_udp_port[udp_count_idx++] = 5353;
	i4_ret = iwpriv_main(4,argv);
	if(i4_ret < 0)
	{
		ALOGE("XXXXXset default udp v4 port 5353 error");
	}
	
	/*udp 4 */	 
	i4_ret = _nw_net_get_port(s_udp_file_path, a_udp_port, MAX_NUM-1);
	if(i4_ret != 0)
	{	 
		return -1;
	} 
	n = 0;
	memset(s_tmp, 0, 512);
	snprintf(s_tmp,512, "%s", "wow_udp_port_v4=");
	for(i=0; i<MAX_NUM-1;i++)
	{
		if(a_udp_port[i] > 0)
		{
			char s[10] = {0};
			snprintf(s,10, "%d,", a_udp_port[i]);
			eth_udp_port[udp_count_idx++] = a_udp_port[i];
			strncat(s_tmp, s ,10);
			n++;
				
		}
	}
	if(n > 0)
	{		 
		s_tmp[strlen(s_tmp) -1] ='\0'; //remove last ','
		//c_dbg_stmt("\nTEST:cmd=%s\n", s_cmd); 		
		ALOGD("(WOWL)udp port = %s", s_tmp);
		argv[3] = s_tmp;
		i4_ret = iwpriv_main(4,argv);
		if(i4_ret < 0)
		{
			ALOGE("XXXXXset udp v4 port error");
		}
	}
		
	n = 0;
	memset(s_tmp, 0, sizeof(s_tmp));
	snprintf(s_tmp, 512 ,"%s", "wow_udp_port_v6=");
		i4_ret = _nw_net_get_port(s_udp6_file_path, a_udp6_port, MAX_NUM);
		if(i4_ret != 0)
		{	 
			return -1;
		}
		for(i=0; i<MAX_NUM;i++)
		{
			if(a_udp6_port[i] > 0)
			{
				char s[10] = {0};
				snprintf(s, 10,"%d,", a_udp6_port[i]);
			eth_udp_port[udp_count_idx++] = a_udp6_port[i];
				strncat(s_tmp, s ,10);
				n++;
				
			}
		}
		if(n > 0)
		{		 
			s_tmp[strlen(s_tmp) -1] ='\0'; //remove last ','
			//c_dbg_stmt("\nTEST:cmd=%s\n", s_cmd); 		
			ALOGD("(WOWL)udp v6 port = %s", s_tmp);
			argv[3] = s_tmp;
			i4_ret = iwpriv_main(4,argv);
			if(i4_ret < 0)
			{
				ALOGE("XXXXXset udp v6 port error");
			}
		} 

	eth_wopacket_para.protocol_type = PROTOCOL_UDP;
	eth_wopacket_para.port_count = udp_count_idx;
	eth_wopacket_para.port_array = eth_udp_port;

	for(k =0;k < udp_count_idx;k++)
	{
		ALOGE("XXXXXethernet udp port%d is %d",k,eth_wopacket_para.port_array[k]);
	}
	i4_ret = ifc_set_Ethernet_listen_port("eth0",&eth_wopacket_para);
	if(i4_ret < 0)
	{
		ALOGE("XXXXXethernet wop failed to set udp_v6 port");
	}
		/*set arp ip*/
		//set ARP offload
		i4_ret = _getlocalIp("wlan0", s_local_ip); 
		if(i4_ret == 0)
		{
			memset(s_tmp, 0, 512);	  
			snprintf(s_tmp,512, "arp_ip=%s", s_local_ip);
			ALOGD("(ARP)wlan0 IP = %s", s_tmp);
			argv[3] = s_tmp;
			i4_ret = iwpriv_main(4,argv);
			if(i4_ret < 0)
			{
				ALOGE("XXXXXset ARP offload error");
			}
		}
		return i4_ret;
#if 0
		const char s_tcp_file_path[] = "/proc/net/tcp";
		const char s_udp_file_path[] = "/proc/net/udp";
		const char s_tcp6_file_path[] = "/proc/net/tcp6";
		const char s_udp6_file_path[] = "/proc/net/udp6";
    #define MAX_NUM (10)
		int a_tcp_port[MAX_NUM + 1] = {0};
		int a_udp_port[MAX_NUM + 1] = {0};
		int a_tcp6_port[MAX_NUM + 1] = {0};
		int a_tcp6_port2[MAX_NUM + 1] = {0};
		int a_udp6_port[MAX_NUM + 1] = {0};
		
		char  s_local_ip[20]		  = {0};
		int i4_ret					= 0;
		char  s_cmd[512]			  = {0};
		int i						= 0;
		int n						= 0;
		int m						= 0;
		//i4_ret = _getlocalIp("wlan0", s_local_ip);
		
		_nw_net_get_port(s_udp_file_path, a_udp_port, MAX_NUM);
	
		/*set tcp ip*/
		sprintf(s_cmd, "%s", "wpa_cli -iwlan0 driver SET_WOW_IP ");
		strncat(s_cmd, "0.0.0.0", 20);//set ip to any
		system(s_cmd);
		//c_dbg_stmt("\nTEST:cmd=%s\n", s_cmd);
		   
		/*set tcp port*/
		i4_ret = _nw_net_get_port(s_tcp_file_path, a_tcp_port, MAX_NUM);
		if(i4_ret != 0)
		{	 
			return -1;
		}
		memset(s_cmd, 0, sizeof(s_cmd));
		sprintf(s_cmd, "%s", "iwpriv wlan0 set wow_tcp_port_v4= ");
		n = 0;
		for(i=0; i<MAX_NUM;i++)
		{
			if(a_tcp_port[i] > 0)
			{
				char s[10] = {0};
				sprintf(s, "%d,", a_tcp_port[i]);
				strcat(s_cmd, s);
				n ++;			 
			}
		}
		if(n>0)
		{
			s_cmd[strlen(s_cmd) -1] ='\0'; //remove last ','
			//c_dbg_stmt("\nTEST:cmd=%s\n", s_cmd); 	   
			ALOGD("(WOWL)cmd = %s", s_cmd);
			system(s_cmd);	 
		}
	
		//tcp6
		i4_ret = _nw_net_get_port_ex(s_tcp6_file_path, a_tcp6_port2,a_tcp6_port, MAX_NUM);
		if(i4_ret != 0)
		{	 
			return -1;
		}
		memset(s_cmd, 0, sizeof(s_cmd));
		sprintf(s_cmd, "%s", "iwpriv wlan0 set wow_tcp_port_v6= ");
	   // n = 0;
	   // tcp + tcp6 = MAX_NUM, so , tcp6 is MAX-n
		for(i=0; i<MAX_NUM -n ;i++)
		{
			if(a_tcp6_port[i] > 0)
			{
				char s[10] = {0};
				sprintf(s, "%d ", a_tcp6_port[i]);
				strcat(s_cmd, s);
				m ++;			 
			}
		}
		if(m>0)
		{
			s_cmd[strlen(s_cmd) -1] ='\0'; //remove last ','
			//c_dbg_stmt("\nTEST:cmd=%s\n", s_cmd); 	   
			ALOGD("(WOWL)cmd = %s", s_cmd);
			system(s_cmd);
		}
	
		//////////////
		memset(s_cmd, 0, sizeof(s_cmd));
		sprintf(s_cmd, "%s", "iwpriv wlan0 set wow_tcp_port_v4= ");
	   // n = 0;
	   // tcp + tcp6 = MAX_NUM, so , tcp6 is MAX-n
		for(i=0; i<MAX_NUM -n ;i++)
		{
			if(a_tcp6_port2[i] > 0)
			{
				char s[10] = {0};
				sprintf(s, "%d ", a_tcp6_port2[i]);
				strcat(s_cmd, s);
				m ++;			 
			}
		}
		if(m>0)
		{
			s_cmd[strlen(s_cmd) -1] ='\0'; //remove last ','
			//c_dbg_stmt("\nTEST:cmd=%s\n", s_cmd); 	   
			ALOGD("(WOWL)cmd = %s", s_cmd);
			system(s_cmd);
		}
	
		 /*set udp port*/
		i4_ret = _nw_net_get_port(s_udp_file_path, a_udp_port, MAX_NUM);
		if(i4_ret != 0)
		{	 
			return -1;
		}  
		memset(s_cmd, 0, sizeof(s_cmd));
		sprintf(s_cmd, "%s", "iwpriv wlan0	wow_udp_port_v4=");
		n = 0;
		m = 0;
		for(i=0; i<MAX_NUM;i++)
		{
			if(a_udp_port[i] > 0)
			{
				char s[10] = {0};
				sprintf(s, "%d,", a_udp_port[i]);
				strcat(s_cmd, s);
				n++;
				
			}
		}
		if(n > 0)
		{		 
			s_cmd[strlen(s_cmd) -1] ='\0'; //remove last ','
			//c_dbg_stmt("\nTEST:cmd=%s\n", s_cmd); 		
			ALOGD("(WOWL)cmd = %s", s_cmd);
			system(s_cmd);
		}
		
		//udp6
		i4_ret = _nw_net_get_port(s_udp6_file_path, a_udp6_port, MAX_NUM);
		if(i4_ret != 0)
		{	 
			return -1;
		}  
		memset(s_cmd, 0, sizeof(s_cmd));
		sprintf(s_cmd, "%s", "iwpriv wlan0	wow_udp_port_v6=");
		//n = 0;
		for(i=0; i<MAX_NUM;i++)
		{
			if(a_udp6_port[i] > 0)
			{
				char s[10] = {0};
				sprintf(s, "%d ", a_udp6_port[i]);
				strcat(s_cmd, s);
				m++;
				
			}
		}
		if(m > 0)
		{		 
			s_cmd[strlen(s_cmd) -1] ='\0'; //remove last ','
			//c_dbg_stmt("\nTEST:cmd=%s\n", s_cmd); 		
			ALOGD("(WOWL)cmd = %s", s_cmd);
			system(s_cmd);
		}	 
	
		//set ARP offload
		_getlocalIp("wlan0", s_local_ip);	 
		memset(s_cmd, 0, sizeof(s_cmd));	
		sprintf(s_cmd, "%s", "iwpriv wlan0 set arp_ip=");
		strcat(s_cmd, s_local_ip);
		system(s_cmd);
		ALOGD("(ARP)cmd = %s", s_cmd);
		
		return 0;
#endif
	
	}

 void main()
{
	INT32       i4_r        = -3;
	INT32		i4_value    = 1;
	char wopacket_status[PROPERTY_VALUE_MAX];
	ALOGE("(yjdbg) 2.power_set_interactive off");
	if (!property_get(WOPACKET_PROP_NAME, wopacket_status, NULL) || strcmp(wopacket_status, "0") == 0) {
		ALOGD ("wopacket Sony behavior");
		/*i4_r = a_mtktvapi_config_get_value(1,CFG_NETWORK_WAKE_ON_PACKET,&i4_value);
		if(MTKTVAPIR_OK == i4_r)
		{
			ALOGD ("succes to call a_mtktvapi_config_get_value");
			
		}else
		{
			ALOGD ("failed to call a_mtktvapi_config_get_value");	
		} */
		if(i4_value == 1)
		{
			ALOGD ("enable WoPacket before power off");	
			a_nw_wlan_set_ip_and_port_to_driver();
		}
	}else{
		ALOGD ("force enable WoPacket");	
		a_nw_wlan_set_ip_and_port_to_driver();
		}
	
	ALOGE("ACFGGGGGGGGGGGGGGGGGGGGGGGGGGGGG : AM_BRDCST_MSG_PRE_POWER_OFF\n");
    
}

















































